@@ -21,7 +21,6 @@
 #include <Arduino.h>
 #include <user_interface.h>
 #include <core_version.h>
-#include <lwip/init.h>      // LWIP_VERSION_*
 #include <lwipopts.h>       // LWIP_HASH_STR (lwip2)
 #include <bearssl/bearssl_git.h>  // BEARSSL_GIT short hash
 
@@ -29,13 +28,11 @@
 #define STR(x) STRHELPER(x) // stringifier
 
 static const char arduino_esp8266_git_ver [] PROGMEM = "/Core:" STR(ARDUINO_ESP8266_GIT_DESC) "=";
-#if LWIP_VERSION_MAJOR > 1
 #if LWIP_IPV6
 static const char lwip_version [] PROGMEM = "/lwIP:IPv6+" LWIP_HASH_STR;
 #else
 static const char lwip_version [] PROGMEM = "/lwIP:" LWIP_HASH_STR;
 #endif
-#endif
 static const char bearssl_version [] PROGMEM = "/BearSSL:" STR(BEARSSL_GIT);
 
 String EspClass::getFullVersion() {
@@ -45,23 +42,7 @@ String EspClass::getFullVersion() {
     s += system_get_sdk_version();
     s += FPSTR(arduino_esp8266_git_ver);
     s += String(esp8266::coreVersionNumeric());
-#if LWIP_VERSION_MAJOR == 1
-    s += F("/lwIP:");
-    s += LWIP_VERSION_MAJOR;
-    s += '.';
-    s += LWIP_VERSION_MINOR;
-    s += '.';
-    s += LWIP_VERSION_REVISION;
-#if LWIP_VERSION_IS_DEVELOPMENT
-    s += F("-dev");
-#endif
-#if LWIP_VERSION_IS_RC
-    s += F("rc");
-    s += String(LWIP_VERSION_RC);
-#endif
-#else // LWIP_VERSION_MAJOR != 1
     s += FPSTR(lwip_version);
-#endif // LWIP_VERSION_MAJOR != 1
     s += FPSTR(bearssl_version);
 
     return s;
