@@ -23,6 +23,8 @@
  $Id$
  */
 
+#include "Arduino.h"
+
 extern "C" {
 #include <stdlib.h>
 }
@@ -77,10 +79,10 @@ long map(long x, long in_min, long in_max, long out_min, long out_max) {
     return (delta * dividend + (divisor / 2)) / divisor + out_min;
 }
 
-unsigned int makeWord(unsigned int w) {
+uint16_t makeWord(uint16_t w) {
     return w;
 }
 
-unsigned int makeWord(unsigned char h, unsigned char l) {
+uint16_t makeWord(byte h, byte l) {
     return (h << 8) | l;
 }
