@@ -12,15 +12,13 @@ extern "C" {
 #include <stdint.h>
 #include <cont.h> // g_pcont declaration
 
-extern bool timeshift64_is_set;
-extern uint32_t sntp_real_timestamp;
-
 bool can_yield();
 void esp_yield();
 void esp_schedule();
 void tune_timeshift64 (uint64_t now_us);
 void disable_extra4k_at_link_time (void) __attribute__((noinline));
 bool sntp_set_timezone_in_seconds(int32_t timezone);
+void __disableWiFiAtBootTime (void) __attribute__((noinline));
 void __real_system_restart_local() __attribute__((noreturn));
 
 uint32_t sqrt32 (uint32_t n);
@@ -31,11 +29,12 @@ uint32_t crc32 (const void* data, size_t length, uint32_t crc = 0xffffffff);
 
 #include <functional>
 
+using BoolCB = std::function<void(bool)>;
 using TrivialCB = std::function<void()>;
 
-void settimeofday_cb (TrivialCB&& cb);
+void settimeofday_cb (const BoolCB& cb);
 void settimeofday_cb (const TrivialCB& cb);
 
-#endif
+#endif // __cplusplus
 
 #endif // __COREDECLS_H
