@@ -115,7 +115,7 @@ struct uart_
 
 
 // called by ISR
-inline size_t ICACHE_RAM_ATTR
+inline size_t IRAM_ATTR
 uart_rx_fifo_available(const int uart_nr)
 {
     return (USS(uart_nr) >> USRXC) & 0xFF;
@@ -144,7 +144,7 @@ uart_rx_available_unsafe(uart_t* uart)
 
 // Copy all the rx fifo bytes that fit into the rx buffer
 // called by ISR
-inline void ICACHE_RAM_ATTR
+inline void IRAM_ATTR
 uart_rx_copy_fifo_to_buffer_unsafe(uart_t* uart)
 {
     struct uart_rx_buffer_ *rx_buffer = uart->rx_buffer;
@@ -240,6 +240,41 @@ uart_peek_char(uart_t* uart)
     return ret;
 }
 
+// return number of byte accessible by uart_peek_buffer()
+size_t uart_peek_available (uart_t* uart)
+{
+    // path for further optimization:
+    // - return already copied buffer pointer (= older data)
+    // - or return fifo when buffer is empty but then any move from fifo to
+    //   buffer should be blocked until peek_consume is called
+
+    ETS_UART_INTR_DISABLE();
+    uart_rx_copy_fifo_to_buffer_unsafe(uart);
+    auto rpos = uart->rx_buffer->rpos;
+    auto wpos = uart->rx_buffer->wpos;
+    ETS_UART_INTR_ENABLE();
+    if(wpos < rpos)
+        return uart->rx_buffer->size - rpos;
+    return wpos - rpos;
+}
+
+// return a pointer to available data buffer (size = available())
+// semantic forbids any kind of read() between peekBuffer() and peekConsume()
+const char* uart_peek_buffer (uart_t* uart)
+{
+    return (const char*)&uart->rx_buffer->buffer[uart->rx_buffer->rpos];
+}
+
+// consume bytes after use (see uart_peek_buffer)
+void uart_peek_consume (uart_t* uart, size_t consume)
+{
+    ETS_UART_INTR_DISABLE();
+    uart->rx_buffer->rpos += consume;
+    if (uart->rx_buffer->rpos >= uart->rx_buffer->size)
+        uart->rx_buffer->rpos -= uart->rx_buffer->size;
+    ETS_UART_INTR_ENABLE();
+}
+
 int
 uart_read_char(uart_t* uart)
 {
@@ -289,7 +324,7 @@ uart_read(uart_t* uart, char* userbuffer, size_t usersize)
 // instead of the uart_isr...uart_rx_copy_fifo_to_buffer_unsafe()
 // Since we've already read the bytes from the FIFO, can't use that
 // function directly and need to implement it bytewise here
-static void ICACHE_RAM_ATTR uart_isr_handle_data(void* arg, uint8_t data)
+static void IRAM_ATTR uart_isr_handle_data(void* arg, uint8_t data)
 {
     uart_t* uart = (uart_t*)arg;
     if(uart == NULL || !uart->rx_enabled) {
@@ -370,9 +405,10 @@ uart_get_rx_buffer_size(uart_t* uart)
 }
 
 // The default ISR handler called when GDB is not enabled
-void ICACHE_RAM_ATTR
-uart_isr(void * arg)
+void IRAM_ATTR
+uart_isr(void * arg, void * frame)
 {
+    (void) frame;
     uart_t* uart = (uart_t*)arg;
     uint32_t usis = USIS(uart->uart_nr);
 
@@ -505,8 +541,10 @@ uart_write(uart_t* uart, const char* buf, size_t size)
 
     size_t ret = size;
     const int uart_nr = uart->uart_nr;
-    while (size--)
+    while (size--) {
         uart_do_write_char(uart_nr, pgm_read_byte(buf++));
+        optimistic_yield(10000UL);
+    }
 
     return ret;
 }
@@ -722,11 +760,11 @@ uart_uninit(uart_t* uart)
     free(uart);
 }
 
-void
+bool
 uart_swap(uart_t* uart, int tx_pin)
 {
     if(uart == NULL)
-        return;
+        return false;
 
     switch(uart->uart_nr)
     {
@@ -737,19 +775,17 @@ uart_swap(uart_t* uart, int tx_pin)
             {
                 pinMode(uart->tx_pin, INPUT);
                 uart->tx_pin = 15;
+                pinMode(uart->tx_pin, FUNCTION_4);
             }
             if(uart->rx_enabled) //RX
             {
                 pinMode(uart->rx_pin, INPUT);
                 uart->rx_pin = 13;
+                pinMode(uart->rx_pin, FUNCTION_4);
             }
-            if(uart->tx_enabled)
-                pinMode(uart->tx_pin, FUNCTION_4);    //TX
-
-            if(uart->rx_enabled)
-                pinMode(uart->rx_pin, FUNCTION_4);    //RX
 
             IOSWAP |= (1 << IOSWAPU0);
+            return true;
         }
         else
         {
@@ -757,19 +793,17 @@ uart_swap(uart_t* uart, int tx_pin)
             {
                 pinMode(uart->tx_pin, INPUT);
                 uart->tx_pin = (tx_pin == 2)?2:1;
+                pinMode(uart->tx_pin, (tx_pin == 2)?FUNCTION_4:SPECIAL);
             }
             if(uart->rx_enabled) //RX
             {
                 pinMode(uart->rx_pin, INPUT);
                 uart->rx_pin = 3;
+                pinMode(3, SPECIAL);
             }
-            if(uart->tx_enabled)
-                pinMode(uart->tx_pin, (tx_pin == 2)?FUNCTION_4:SPECIAL);    //TX
-
-            if(uart->rx_enabled)
-                pinMode(3, SPECIAL);    //RX
 
             IOSWAP &= ~(1 << IOSWAPU0);
+            return true;
         }
         break;
     case UART1:
@@ -778,30 +812,30 @@ uart_swap(uart_t* uart, int tx_pin)
     default:
         break;
     }
+    return false;
 }
 
-void
+bool
 uart_set_tx(uart_t* uart, int tx_pin)
 {
     if(uart == NULL)
-        return;
+        return false;
 
     switch(uart->uart_nr)
     {
     case UART0:
         if(uart->tx_enabled)
         {
-            if (uart->tx_pin == 1 && tx_pin == 2)
+            if (uart->tx_pin == tx_pin)
             {
-                pinMode(uart->tx_pin, INPUT);
-                uart->tx_pin = 2;
-                pinMode(uart->tx_pin, FUNCTION_4);
+                return true;
             }
-            else if (uart->tx_pin == 2 && tx_pin != 2)
+            else if (tx_pin == 1 || tx_pin == 2)
             {
                 pinMode(uart->tx_pin, INPUT);
-                uart->tx_pin = 1;
-                pinMode(uart->tx_pin, SPECIAL);
+                uart->tx_pin = tx_pin;
+                pinMode(uart->tx_pin, tx_pin == 1 ? SPECIAL : FUNCTION_4);
+                return true;
             }
         }
 
@@ -812,33 +846,54 @@ uart_set_tx(uart_t* uart, int tx_pin)
     default:
         break;
     }
+    return false;
 }
 
-void
+bool
 uart_set_pins(uart_t* uart, int tx, int rx)
 {
     if(uart == NULL)
-        return;
+        return false;
 
-    if(uart->uart_nr == UART0) // Only UART0 allows pin changes
+    if(uart->uart_nr != UART0) // Only UART0 allows pin changes
+        return false;
+
+    if(uart->tx_enabled && uart->tx_pin != tx)
     {
-        if(uart->tx_enabled && uart->tx_pin != tx)
+        if( rx == 13 && tx == 15)
         {
-            if( rx == 13 && tx == 15)
+            if (!uart_swap(uart, 15))
+                return false;
+        }
+        else if (rx == 3 && (tx == 1 || tx == 2))
+        {
+            if (uart->rx_pin != rx)
             {
-                uart_swap(uart, 15);
+                if (!uart_swap(uart, tx))
+                    return false;
             }
-            else if (rx == 3 && (tx == 1 || tx == 2))
+            else
             {
-                if (uart->rx_pin != rx)
-                    uart_swap(uart, tx);
-                else
-                    uart_set_tx(uart, tx);
+                if (!uart_set_tx(uart, tx))
+                    return false;
             }
         }
-        if(uart->rx_enabled && uart->rx_pin != rx && rx == 13 && tx == 15)
-            uart_swap(uart, 15);
+        else
+            return false;
     }
+
+    if (uart->rx_enabled && uart->rx_pin != rx)
+    {
+        if (rx == 13 && tx == 15)
+        {
+            if (!uart_swap(uart, 15))
+                return false;
+        }
+        else
+            return false;
+    }
+
+    return true;
 }
 
 
