@@ -178,8 +178,7 @@ void DNSServer::processNextRequest()
     return;
 
   std::unique_ptr<uint8_t[]> buffer(new (std::nothrow) uint8_t[currentPacketSize]);
-
-  if (buffer == NULL)
+  if (buffer == nullptr)
     return;
 
   _udp.read(buffer.get(), currentPacketSize);
