@@ -46,6 +46,9 @@ class UdpContext;
 #define SSDP_MODEL_VERSION_SIZE     32
 #define SSDP_MANUFACTURER_SIZE      64
 #define SSDP_MANUFACTURER_URL_SIZE  128
+#define SSDP_INTERVAL_SECONDS       1200
+#define SSDP_MULTICAST_TTL          2
+#define SSDP_HTTP_PORT              80
 
 typedef enum {
   NONE,
@@ -101,20 +104,20 @@ class SSDPClass{
     void _stopTimer();
     static void _onTimerStatic(SSDPClass* self);
 
-    UdpContext* _server;
-    SSDPTimer* _timer;
-    uint16_t _port;
-    uint8_t _ttl;
-    uint32_t _interval;
+    UdpContext* _server = nullptr;
+    SSDPTimer* _timer = nullptr;
+    uint16_t _port = SSDP_HTTP_PORT;
+    uint8_t _ttl = SSDP_MULTICAST_TTL;
+    uint32_t _interval = SSDP_INTERVAL_SECONDS;
 
     IPAddress _respondToAddr;
-    uint16_t  _respondToPort;
+    uint16_t  _respondToPort = 0;
 
-    bool _pending;
-    bool _st_is_uuid;
-    unsigned short _delay;
-    unsigned long _process_time;
-    unsigned long _notify_time;
+    bool _pending = false;
+    bool _st_is_uuid = false;
+    unsigned short _delay = 0;
+    unsigned long _process_time = 0;
+    unsigned long _notify_time = 0;
 
     char _schemaURL[SSDP_SCHEMA_URL_SIZE];
     char _uuid[SSDP_UUID_SIZE];
