@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2011-2018 Bill Greiman
+ * Copyright (c) 2011-2020 Bill Greiman
  * This file is part of the SdFat library for SD memory cards.
  *
  * MIT License
@@ -22,16 +22,26 @@
  * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  * DEALINGS IN THE SOFTWARE.
  */
-#if defined(__MK64FX512__) || defined(__MK66FX1M0__)
+#if defined(__MK64FX512__) || defined(__MK66FX1M0__) || defined(__IMXRT1062__)
+#include "SdioTeensy.h"
+#include "SdCardInfo.h"
 #include "SdioCard.h"
 
+
 namespace sdfat {
 
+
+//==============================================================================
+// limit of K66 due to errata KINETIS_K_0N65N.
+const uint32_t MAX_SDHC_COUNT = 0XFFFF;
+
+// Max RU is 1024 sectors.
+const uint32_t RU_MASK = 0X03FF;
 //==============================================================================
 #define SDHC_PROCTL_DTW_4BIT 0x01
 const uint32_t FIFO_WML = 16;
-const uint32_t CMD8_RETRIES = 10;
-const uint32_t BUSY_TIMEOUT_MICROS = 500000;
+const uint32_t CMD8_RETRIES = 3;
+const uint32_t BUSY_TIMEOUT_MICROS = 1000000;
 //==============================================================================
 const uint32_t SDHC_IRQSTATEN_MASK =
                SDHC_IRQSTATEN_DMAESEN | SDHC_IRQSTATEN_AC12ESEN |
@@ -59,7 +69,7 @@ const uint32_t SDHC_IRQSIGEN_MASK =
                SDHC_IRQSIGEN_DTOEIEN | SDHC_IRQSIGEN_CIEIEN |
                SDHC_IRQSIGEN_CEBEIEN | SDHC_IRQSIGEN_CCEIEN |
                SDHC_IRQSIGEN_CTOEIEN | SDHC_IRQSIGEN_TCIEN;
-//=============================================================================
+//==============================================================================
 const uint32_t CMD_RESP_NONE = SDHC_XFERTYP_RSPTYP(0);
 
 const uint32_t CMD_RESP_R1 = SDHC_XFERTYP_CICEN | SDHC_XFERTYP_CCCEN |
@@ -76,6 +86,7 @@ const uint32_t CMD_RESP_R6 = CMD_RESP_R1;
 
 const uint32_t CMD_RESP_R7 = CMD_RESP_R1;
 
+#if defined(__MK64FX512__) || defined(__MK66FX1M0__)
 const uint32_t DATA_READ = SDHC_XFERTYP_DTDSEL | SDHC_XFERTYP_DPSEL;
 
 const uint32_t DATA_READ_DMA = DATA_READ | SDHC_XFERTYP_DMAEN;
@@ -84,16 +95,44 @@ const uint32_t DATA_READ_MULTI_DMA = DATA_READ_DMA | SDHC_XFERTYP_MSBSEL |
                                      SDHC_XFERTYP_AC12EN | SDHC_XFERTYP_BCEN;
 
 const uint32_t DATA_READ_MULTI_PGM = DATA_READ | SDHC_XFERTYP_MSBSEL |
-                                     SDHC_XFERTYP_BCEN | SDHC_XFERTYP_AC12EN;
+                                     SDHC_XFERTYP_BCEN;
 
 const uint32_t DATA_WRITE_DMA = SDHC_XFERTYP_DPSEL | SDHC_XFERTYP_DMAEN;
 
 const uint32_t DATA_WRITE_MULTI_DMA = DATA_WRITE_DMA | SDHC_XFERTYP_MSBSEL |
                                       SDHC_XFERTYP_AC12EN | SDHC_XFERTYP_BCEN;
 
-const uint32_t DATA_WRITE_MULTI_PGM = SDHC_XFERTYP_DPSEL |
-                                      SDHC_XFERTYP_MSBSEL |
-                                      SDHC_XFERTYP_BCEN | SDHC_XFERTYP_AC12EN;
+const uint32_t DATA_WRITE_MULTI_PGM = SDHC_XFERTYP_DPSEL | SDHC_XFERTYP_MSBSEL |
+                                      SDHC_XFERTYP_BCEN;
+
+#elif defined(__IMXRT1062__)
+// Use low bits for SDHC_MIX_CTRL since bits 15-0 of SDHC_XFERTYP are reserved.
+const uint32_t SDHC_MIX_CTRL_MASK = SDHC_MIX_CTRL_DMAEN | SDHC_MIX_CTRL_BCEN |
+                                    SDHC_MIX_CTRL_AC12EN |
+                                    SDHC_MIX_CTRL_DDR_EN |
+                                    SDHC_MIX_CTRL_DTDSEL |
+                                    SDHC_MIX_CTRL_MSBSEL |
+                                    SDHC_MIX_CTRL_NIBBLE_POS |
+                                    SDHC_MIX_CTRL_AC23EN;
+
+const uint32_t DATA_READ = SDHC_MIX_CTRL_DTDSEL | SDHC_XFERTYP_DPSEL;
+
+const uint32_t DATA_READ_DMA = DATA_READ | SDHC_MIX_CTRL_DMAEN;
+
+const uint32_t DATA_READ_MULTI_DMA = DATA_READ_DMA | SDHC_MIX_CTRL_MSBSEL |
+                                     SDHC_MIX_CTRL_AC12EN | SDHC_MIX_CTRL_BCEN;
+
+const uint32_t DATA_READ_MULTI_PGM = DATA_READ | SDHC_MIX_CTRL_MSBSEL;
+
+
+const uint32_t DATA_WRITE_DMA = SDHC_XFERTYP_DPSEL | SDHC_MIX_CTRL_DMAEN;
+
+const uint32_t DATA_WRITE_MULTI_DMA = DATA_WRITE_DMA | SDHC_MIX_CTRL_MSBSEL |
+                                      SDHC_MIX_CTRL_AC12EN | SDHC_MIX_CTRL_BCEN;
+
+const uint32_t DATA_WRITE_MULTI_PGM = SDHC_XFERTYP_DPSEL | SDHC_MIX_CTRL_MSBSEL;
+
+#endif  // defined(__MK64FX512__) || defined(__MK66FX1M0__)
 
 const uint32_t ACMD6_XFERTYP = SDHC_XFERTYP_CMDINX(ACMD6) | CMD_RESP_R1;
 
@@ -116,6 +155,8 @@ const uint32_t CMD9_XFERTYP = SDHC_XFERTYP_CMDINX(CMD9) | CMD_RESP_R2;
 
 const uint32_t CMD10_XFERTYP = SDHC_XFERTYP_CMDINX(CMD10) | CMD_RESP_R2;
 
+const uint32_t CMD11_XFERTYP = SDHC_XFERTYP_CMDINX(CMD11) | CMD_RESP_R1;
+
 const uint32_t CMD12_XFERTYP = SDHC_XFERTYP_CMDINX(CMD12) | CMD_RESP_R1b |
                                SDHC_XFERTYP_CMDTYP(3);
 
@@ -147,7 +188,7 @@ const uint32_t CMD38_XFERTYP = SDHC_XFERTYP_CMDINX(CMD38) | CMD_RESP_R1b;
 
 const uint32_t CMD55_XFERTYP = SDHC_XFERTYP_CMDINX(CMD55) | CMD_RESP_R1;
 
-//=============================================================================
+//==============================================================================
 static bool cardCommand(uint32_t xfertyp, uint32_t arg);
 static void enableGPIO(bool enable);
 static void enableDmaIrs();
@@ -160,7 +201,7 @@ static void setSdclk(uint32_t kHzMax);
 static bool yieldTimeout(bool (*fcn)());
 static bool waitDmaStatus();
 static bool waitTimeout(bool (*fcn)());
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool (*m_busyFcn)() = 0;
 static bool m_initDone = false;
 static bool m_version2;
@@ -174,7 +215,8 @@ static uint32_t m_sdClkKhz = 0;
 static uint32_t m_ocr;
 static cid_t m_cid;
 static csd_t m_csd;
-//=============================================================================
+//==============================================================================
+#define DBG_TRACE Serial.print("TRACE."); Serial.println(__LINE__); delay(200);
 #define USE_DEBUG_MODE 0
 #if USE_DEBUG_MODE
 #define DBG_IRQSTAT() if (SDHC_IRQSTAT) {Serial.print(__LINE__);\
@@ -182,6 +224,10 @@ static csd_t m_csd;
 
 static void printRegs(uint32_t line) {
   Serial.print(line);
+  Serial.print(" SDHC_BLKATTR ");
+  Serial.print(SDHC_BLKATTR, HEX);
+  Serial.print(" XFERTYP ");
+  Serial.print(SDHC_XFERTYP, HEX);
   Serial.print(" PRSSTAT ");
   Serial.print(SDHC_PRSSTAT, HEX);
   Serial.print(" PROCTL ");
@@ -194,34 +240,131 @@ static void printRegs(uint32_t line) {
 #else  // USE_DEBUG_MODE
 #define DBG_IRQSTAT()
 #endif  // USE_DEBUG_MODE
-//=============================================================================
+//==============================================================================
 // Error function and macro.
 #define sdError(code) setSdErrorCode(code, __LINE__)
 inline bool setSdErrorCode(uint8_t code, uint32_t line) {
   m_errorCode = code;
   m_errorLine = line;
-  return false;  // setSdErrorCode
+#if USE_DEBUG_MODE
+  printRegs(line);
+#endif  // USE_DEBUG_MODE
+  return false;
 }
-//=============================================================================
+//==============================================================================
 // ISR
-void sdhc_isr() {
+static void sdIrs() {
   SDHC_IRQSIGEN = 0;
   m_irqstat = SDHC_IRQSTAT;
   SDHC_IRQSTAT = m_irqstat;
+#if defined(__IMXRT1062__)
+  SDHC_MIX_CTRL &= ~(SDHC_MIX_CTRL_AC23EN | SDHC_MIX_CTRL_DMAEN);
+#endif
   m_dmaBusy = false;
 }
-//=============================================================================
+//==============================================================================
+// GPIO and clock functions.
+#if defined(__MK64FX512__) || defined(__MK66FX1M0__)
+//------------------------------------------------------------------------------
+static void enableGPIO(bool enable) {
+  const uint32_t PORT_CLK = PORT_PCR_MUX(4) | PORT_PCR_DSE;
+  const uint32_t PORT_CMD_DATA = PORT_CLK   | PORT_PCR_PE | PORT_PCR_PS;
+  const uint32_t PORT_PUP = PORT_PCR_MUX(1) | PORT_PCR_PE | PORT_PCR_PS;
+
+  PORTE_PCR0 = enable ? PORT_CMD_DATA : PORT_PUP;  // SDHC_D1
+  PORTE_PCR1 = enable ? PORT_CMD_DATA : PORT_PUP;  // SDHC_D0
+  PORTE_PCR2 = enable ? PORT_CLK      : PORT_PUP;  // SDHC_CLK
+  PORTE_PCR3 = enable ? PORT_CMD_DATA : PORT_PUP;  // SDHC_CMD
+  PORTE_PCR4 = enable ? PORT_CMD_DATA : PORT_PUP;  // SDHC_D3
+  PORTE_PCR5 = enable ? PORT_CMD_DATA : PORT_PUP;  // SDHC_D2
+}
+//------------------------------------------------------------------------------
+static void initClock() {
+#ifdef HAS_KINETIS_MPU
+  // Allow SDHC Bus Master access.
+  MPU_RGDAAC0 |= 0x0C000000;
+#endif  // HAS_KINETIS_MPU
+  // Enable SDHC clock.
+  SIM_SCGC3 |= SIM_SCGC3_SDHC;
+}
+static uint32_t baseClock() { return F_CPU;}
+
+#elif defined(__IMXRT1062__)
+//------------------------------------------------------------------------------
+static void gpioMux(uint8_t mode) {
+  IOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B0_04 = mode;  // DAT2
+  IOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B0_05 = mode;  // DAT3
+  IOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B0_00 = mode;  // CMD
+  IOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B0_01 = mode;  // CLK
+  IOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B0_02 = mode;  // DAT0
+  IOMUXC_SW_MUX_CTL_PAD_GPIO_SD_B0_03 = mode;  // DAT1
+}
+//------------------------------------------------------------------------------
+// add speed strength args?
+static void enableGPIO(bool enable) {
+  const uint32_t CLOCK_MASK = IOMUXC_SW_PAD_CTL_PAD_PKE |
+#if defined(ARDUINO_TEENSY41)
+                              IOMUXC_SW_PAD_CTL_PAD_DSE(1) |
+#else  // defined(ARDUINO_TEENSY41)
+                              IOMUXC_SW_PAD_CTL_PAD_DSE(4) |  ///// WHG
+#endif  // defined(ARDUINO_TEENSY41)
+                              IOMUXC_SW_PAD_CTL_PAD_SPEED(2);
+
+  const uint32_t DATA_MASK = CLOCK_MASK | IOMUXC_SW_PAD_CTL_PAD_PUE |
+                             IOMUXC_SW_PAD_CTL_PAD_PUS(1);
+  if (enable) {
+    gpioMux(0);
+    IOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B0_04 = DATA_MASK;   // DAT2
+    IOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B0_05 = DATA_MASK;   // DAT3
+    IOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B0_00 = DATA_MASK;   // CMD
+    IOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B0_01 = CLOCK_MASK;  // CLK
+    IOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B0_02 = DATA_MASK;   // DAT0
+    IOMUXC_SW_PAD_CTL_PAD_GPIO_SD_B0_03 = DATA_MASK;   // DAT1
+  } else {
+    gpioMux(5);
+  }
+}
+//------------------------------------------------------------------------------
+static void initClock() {
+  /* set PDF_528 PLL2PFD0 */
+  CCM_ANALOG_PFD_528 |= (1 << 7);
+  CCM_ANALOG_PFD_528 &= ~(0x3F << 0);
+  CCM_ANALOG_PFD_528 |= ((24) & 0x3F << 0);  // 12 - 35
+  CCM_ANALOG_PFD_528 &= ~(1 << 7);
+
+  /* Enable USDHC clock. */
+  CCM_CCGR6 |= CCM_CCGR6_USDHC1(CCM_CCGR_ON);
+  CCM_CSCDR1 &= ~(CCM_CSCDR1_USDHC1_CLK_PODF_MASK);
+  CCM_CSCMR1 |= CCM_CSCMR1_USDHC1_CLK_SEL;          // PLL2PFD0
+//  CCM_CSCDR1 |= CCM_CSCDR1_USDHC1_CLK_PODF((7)); / &0x7  WHG
+  CCM_CSCDR1 |= CCM_CSCDR1_USDHC1_CLK_PODF((1));
+}
+//------------------------------------------------------------------------------
+static uint32_t baseClock() {
+  uint32_t divider = ((CCM_CSCDR1 >> 11) & 0x7) + 1;
+  return (528000000U * 3)/((CCM_ANALOG_PFD_528 & 0x3F)/6)/divider;
+}
+#endif  // defined(__MK64FX512__) || defined(__MK66FX1M0__)
+//==============================================================================
 // Static functions.
 static bool cardAcmd(uint32_t rca, uint32_t xfertyp, uint32_t arg) {
   return cardCommand(CMD55_XFERTYP, rca) && cardCommand (xfertyp, arg);
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool cardCommand(uint32_t xfertyp, uint32_t arg) {
   DBG_IRQSTAT();
   if (waitTimeout(isBusyCommandInhibit)) {
     return false;  // Caller will set errorCode.
   }
   SDHC_CMDARG = arg;
+#if defined(__IMXRT1062__)
+  // Set MIX_CTRL if data transfer.
+  if (xfertyp & SDHC_XFERTYP_DPSEL) {
+    SDHC_MIX_CTRL &= ~SDHC_MIX_CTRL_MASK;
+    SDHC_MIX_CTRL |= xfertyp & SDHC_MIX_CTRL_MASK;
+  }
+  xfertyp &= ~SDHC_MIX_CTRL_MASK;
+#endif  // defined(__IMXRT1062__)
   SDHC_XFERTYP = xfertyp;
   if (waitTimeout(isBusyCommandComplete)) {
     return false;  // Caller will set errorCode.
@@ -232,7 +375,7 @@ static bool cardCommand(uint32_t xfertyp, uint32_t arg) {
   return (m_irqstat & SDHC_IRQSTAT_CC) &&
          !(m_irqstat & SDHC_IRQSTAT_CMD_ERROR);
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool cardCMD6(uint32_t arg, uint8_t* status) {
   // CMD6 returns 64 bytes.
   if (waitTimeout(isBusyCMD13)) {
@@ -240,57 +383,47 @@ static bool cardCMD6(uint32_t arg, uint8_t* status) {
   }
   enableDmaIrs();
   SDHC_DSADDR  = (uint32_t)status;
-  SDHC_CMDARG = arg;
   SDHC_BLKATTR = SDHC_BLKATTR_BLKCNT(1) | SDHC_BLKATTR_BLKSIZE(64);
   SDHC_IRQSIGEN = SDHC_IRQSIGEN_MASK;
-  SDHC_XFERTYP = CMD6_XFERTYP;
-
-  if (!waitDmaStatus()) {
+  if (!cardCommand(CMD6_XFERTYP, arg)) {
     return sdError(SD_CARD_ERROR_CMD6);
   }
+  if (!waitDmaStatus()) {
+    return sdError(SD_CARD_ERROR_DMA);
+  }
   return true;
 }
-//-----------------------------------------------------------------------------
-static void enableGPIO(bool enable) {
-  const uint32_t PORT_CLK = PORT_PCR_MUX(4) | PORT_PCR_DSE;
-  const uint32_t PORT_CMD_DATA = PORT_CLK | PORT_PCR_PS | PORT_PCR_PE;
-
-  PORTE_PCR0 = enable ? PORT_CMD_DATA : 0;  // SDHC_D1
-  PORTE_PCR1 = enable ? PORT_CMD_DATA : 0;  // SDHC_D0
-  PORTE_PCR2 = enable ? PORT_CLK : 0;       // SDHC_CLK
-  PORTE_PCR3 = enable ? PORT_CMD_DATA : 0;  // SDHC_CMD
-  PORTE_PCR4 = enable ? PORT_CMD_DATA : 0;  // SDHC_D3
-  PORTE_PCR5 = enable ? PORT_CMD_DATA : 0;  // SDHC_D2
-}
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static void enableDmaIrs() {
   m_dmaBusy = true;
   m_irqstat = 0;
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static void initSDHC() {
-#ifdef HAS_KINETIS_MPU
-  // Allow SDHC Bus Master access.
-  MPU_RGDAAC0 |= 0x0C000000;
-#endif
-  // Enable SDHC clock.
-  SIM_SCGC3 |= SIM_SCGC3_SDHC;
+  initClock();
 
   // Disable GPIO clock.
   enableGPIO(false);
 
+#if defined (__IMXRT1062__)
+  SDHC_MIX_CTRL |= 0x80000000;
+#endif
+
   // Reset SDHC. Use default Water Mark Level of 16.
-  SDHC_SYSCTL = SDHC_SYSCTL_RSTA;
+  SDHC_SYSCTL |= SDHC_SYSCTL_RSTA | SDHC_SYSCTL_SDCLKFS(0x80);
+
   while (SDHC_SYSCTL & SDHC_SYSCTL_RSTA) {
   }
+
   // Set initial SCK rate.
-  setSdclk(400);
+  setSdclk(SD_MAX_INIT_RATE_KHZ);
 
   enableGPIO(true);
 
   // Enable desired IRQSTAT bits.
   SDHC_IRQSTATEN = SDHC_IRQSTATEN_MASK;
 
+  attachInterruptVector(IRQ_SDHC, sdIrs);
   NVIC_SET_PRIORITY(IRQ_SDHC, 6*16);
   NVIC_ENABLE_IRQ(IRQ_SDHC);
 
@@ -299,7 +432,11 @@ static void initSDHC() {
   while (SDHC_SYSCTL & SDHC_SYSCTL_INITA) {
   }
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
+static uint32_t statusCMD13() {
+  return cardCommand(CMD13_XFERTYP, m_rca) ? SDHC_CMDRSP0 : 0;
+}
+//------------------------------------------------------------------------------
 static bool isBusyCMD13() {
   if (!cardCommand(CMD13_XFERTYP, m_rca)) {
     // Caller will timeout.
@@ -307,33 +444,33 @@ static bool isBusyCMD13() {
   }
   return !(SDHC_CMDRSP0 & CARD_STATUS_READY_FOR_DATA);
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool isBusyCommandComplete() {
-  return !(SDHC_IRQSTAT &(SDHC_IRQSTAT_CC | SDHC_IRQSTAT_CMD_ERROR));
+  return !(SDHC_IRQSTAT & (SDHC_IRQSTAT_CC | SDHC_IRQSTAT_CMD_ERROR));
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool isBusyCommandInhibit() {
   return SDHC_PRSSTAT & SDHC_PRSSTAT_CIHB;
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool isBusyDMA() {
   return m_dmaBusy;
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool isBusyFifoRead() {
   return !(SDHC_PRSSTAT & SDHC_PRSSTAT_BREN);
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool isBusyFifoWrite() {
   return !(SDHC_PRSSTAT & SDHC_PRSSTAT_BWEN);
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool isBusyTransferComplete() {
   return !(SDHC_IRQSTAT & (SDHC_IRQSTAT_TC | SDHC_IRQSTAT_ERROR));
 }
-//-----------------------------------------------------------------------------
-static bool rdWrBlocks(uint32_t xfertyp,
-                       uint32_t lba, uint8_t* buf, size_t n) {
+//------------------------------------------------------------------------------
+static bool rdWrSectors(uint32_t xfertyp,
+                       uint32_t sector, uint8_t* buf, size_t n) {
   if ((3 & (uint32_t)buf) || n == 0) {
     return sdError(SD_CARD_ERROR_DMA);
   }
@@ -342,14 +479,14 @@ static bool rdWrBlocks(uint32_t xfertyp,
   }
   enableDmaIrs();
   SDHC_DSADDR  = (uint32_t)buf;
-  SDHC_CMDARG = m_highCapacity ? lba : 512*lba;
   SDHC_BLKATTR = SDHC_BLKATTR_BLKCNT(n) | SDHC_BLKATTR_BLKSIZE(512);
   SDHC_IRQSIGEN = SDHC_IRQSIGEN_MASK;
-  SDHC_XFERTYP = xfertyp;
-
+  if (!cardCommand(xfertyp, m_highCapacity ? sector : 512*sector)) {
+    return false;
+  }
   return waitDmaStatus();
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // Read 16 byte CID or CSD register.
 static bool readReg16(uint32_t xfertyp, void* data) {
   uint8_t* d = reinterpret_cast<uint8_t*>(data);
@@ -363,26 +500,28 @@ static bool readReg16(uint32_t xfertyp, void* data) {
   d[15] = 0;
   return true;
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static void setSdclk(uint32_t kHzMax) {
   const uint32_t DVS_LIMIT = 0X10;
   const uint32_t SDCLKFS_LIMIT = 0X100;
   uint32_t dvs = 1;
   uint32_t sdclkfs = 1;
   uint32_t maxSdclk = 1000*kHzMax;
+  uint32_t base = baseClock();
 
-  while ((F_CPU/(sdclkfs*DVS_LIMIT) > maxSdclk) && (sdclkfs < SDCLKFS_LIMIT)) {
+  while ((base/(sdclkfs*DVS_LIMIT) > maxSdclk) && (sdclkfs < SDCLKFS_LIMIT)) {
     sdclkfs <<= 1;
   }
-  while ((F_CPU/(sdclkfs*dvs) > maxSdclk) && (dvs < DVS_LIMIT)) {
+  while ((base/(sdclkfs*dvs) > maxSdclk) && (dvs < DVS_LIMIT)) {
     dvs++;
   }
-  m_sdClkKhz = F_CPU/(1000*sdclkfs*dvs);
+  m_sdClkKhz = base/(1000*sdclkfs*dvs);
   sdclkfs >>= 1;
   dvs--;
-
+#if defined(__MK64FX512__) || defined(__MK66FX1M0__)
   // Disable SDHC clock.
   SDHC_SYSCTL &= ~SDHC_SYSCTL_SDCLKEN;
+#endif  // defined(__MK64FX512__) || defined(__MK66FX1M0__)
 
   // Change dividers.
   uint32_t sysctl = SDHC_SYSCTL & ~(SDHC_SYSCTL_DTOCV_MASK
@@ -394,13 +533,14 @@ static void setSdclk(uint32_t kHzMax) {
   // Wait until the SDHC clock is stable.
   while (!(SDHC_PRSSTAT & SDHC_PRSSTAT_SDSTB)) {
   }
+
+#if defined(__MK64FX512__) || defined(__MK66FX1M0__)
   // Enable the SDHC clock.
   SDHC_SYSCTL |= SDHC_SYSCTL_SDCLKEN;
+#endif  // defined(__MK64FX512__) || defined(__MK66FX1M0__)
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool transferStop() {
-  DBG_IRQSTAT();
-
   if (!cardCommand(CMD12_XFERTYP, 0)) {
     return sdError(SD_CARD_ERROR_CMD12);
   }
@@ -410,17 +550,14 @@ static bool transferStop() {
   // Save registers before reset DAT lines.
   uint32_t irqsststen = SDHC_IRQSTATEN;
   uint32_t proctl = SDHC_PROCTL & ~SDHC_PROCTL_SABGREQ;
-
   // Do reset to clear CDIHB.  Should be a better way!
   SDHC_SYSCTL |= SDHC_SYSCTL_RSTD;
-
   // Restore registers.
   SDHC_IRQSTATEN = irqsststen;
   SDHC_PROCTL = proctl;
-
   return true;
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // Return true if timeout occurs.
 static bool yieldTimeout(bool (*fcn)()) {
   m_busyFcn = fcn;
@@ -430,19 +567,19 @@ static bool yieldTimeout(bool (*fcn)()) {
       m_busyFcn = 0;
       return true;
     }
-    yield();
+    SysCall::yield();
   }
   m_busyFcn = 0;
   return false;  // Caller will set errorCode.
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 static bool waitDmaStatus() {
   if (yieldTimeout(isBusyDMA)) {
     return false;  // Caller will set errorCode.
   }
   return (m_irqstat & SDHC_IRQSTAT_TC) && !(m_irqstat & SDHC_IRQSTAT_ERROR);
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 // Return true if timeout occurs.
 static bool waitTimeout(bool (*fcn)()) {
   uint32_t m = micros();
@@ -453,10 +590,14 @@ static bool waitTimeout(bool (*fcn)()) {
   }
   return false;  // Caller will set errorCode.
 }
-//=============================================================================
-bool SdioCard::begin() {
+//==============================================================================
+// Start of SdioCard member functions.
+//==============================================================================
+bool SdioCard::begin(SdioConfig sdioConfig) {
   uint32_t kHzSdClk;
   uint32_t arg;
+  m_sdioConfig = sdioConfig;
+  m_curState = IDLE_STATE;
   m_initDone = false;
   m_errorCode = SD_CARD_ERROR_NONE;
   m_highCapacity = false;
@@ -464,7 +605,6 @@ bool SdioCard::begin() {
 
   // initialize controller.
   initSDHC();
-
   if (!cardCommand(CMD0_XFERTYP, 0)) {
     return sdError(SD_CARD_ERROR_CMD0);
   }
@@ -479,14 +619,13 @@ bool SdioCard::begin() {
     }
   }
   arg = m_version2 ? 0X40300000 : 0x00300000;
-  uint32_t m = micros();
+  int m = micros();
   do {
     if (!cardAcmd(0, ACMD41_XFERTYP, arg) ||
        ((micros() - m) > BUSY_TIMEOUT_MICROS)) {
       return sdError(SD_CARD_ERROR_ACMD41);
     }
   } while ((SDHC_CMDRSP0 & 0x80000000) == 0);
-
   m_ocr = SDHC_CMDRSP0;
   if (SDHC_CMDRSP0 & 0x40000000) {
     // Is high capacity.
@@ -520,6 +659,7 @@ bool SdioCard::begin() {
   SDHC_WML = SDHC_WML_RDWML(FIFO_WML) | SDHC_WML_WRWML(FIFO_WML);
 
   // Determine if High Speed mode is supported and set frequency.
+  // Check status[16] for error 0XF or status[16] for new mode 0X1.
   uint8_t status[64];
   if (cardCMD6(0X00FFFFFF, status) && (2 & status[13]) &&
       cardCMD6(0X80FFFFF1, status) && (status[16] & 0XF) == 1) {
@@ -527,40 +667,36 @@ bool SdioCard::begin() {
   } else {
     kHzSdClk = 25000;
   }
-  // disable GPIO
+  // Disable GPIO.
   enableGPIO(false);
 
   // Set the SDHC SCK frequency.
   setSdclk(kHzSdClk);
 
-  // enable GPIO
+  // Enable GPIO.
   enableGPIO(true);
   m_initDone = true;
   return true;
 }
-//-----------------------------------------------------------------------------
-uint32_t SdioCard::cardCapacity() {
-  return sdCardCapacity(&m_csd);
-}
-//-----------------------------------------------------------------------------
-bool SdioCard::erase(uint32_t firstBlock, uint32_t lastBlock) {
-  // check for single block erase
+//------------------------------------------------------------------------------
+bool SdioCard::erase(uint32_t firstSector, uint32_t lastSector) {
+  // check for single sector erase
   if (!m_csd.v1.erase_blk_en) {
     // erase size mask
     uint8_t m = (m_csd.v1.sector_size_high << 1) | m_csd.v1.sector_size_low;
-    if ((firstBlock & m) != 0 || ((lastBlock + 1) & m) != 0) {
+    if ((firstSector & m) != 0 || ((lastSector + 1) & m) != 0) {
       // error card can't erase specified area
-      return sdError(SD_CARD_ERROR_ERASE_SINGLE_BLOCK);
+      return sdError(SD_CARD_ERROR_ERASE_SINGLE_SECTOR);
     }
   }
   if (!m_highCapacity) {
-    firstBlock <<= 9;
-    lastBlock <<= 9;
+    firstSector <<= 9;
+    lastSector <<= 9;
   }
-  if (!cardCommand(CMD32_XFERTYP, firstBlock)) {
+  if (!cardCommand(CMD32_XFERTYP, firstSector)) {
     return sdError(SD_CARD_ERROR_CMD32);
   }
-  if (!cardCommand(CMD33_XFERTYP, lastBlock)) {
+  if (!cardCommand(CMD33_XFERTYP, lastSector)) {
      return sdError(SD_CARD_ERROR_CMD33);
   }
   if (!cardCommand(CMD38_XFERTYP, 0)) {
@@ -571,81 +707,47 @@ bool SdioCard::erase(uint32_t firstBlock, uint32_t lastBlock) {
   }
   return true;
 }
-//-----------------------------------------------------------------------------
-uint8_t SdioCard::errorCode() {
+//------------------------------------------------------------------------------
+uint8_t SdioCard::errorCode() const {
   return m_errorCode;
 }
-//-----------------------------------------------------------------------------
-uint32_t SdioCard::errorData() {
+//------------------------------------------------------------------------------
+uint32_t SdioCard::errorData() const {
   return m_irqstat;
 }
-//-----------------------------------------------------------------------------
-uint32_t SdioCard::errorLine() {
+//------------------------------------------------------------------------------
+uint32_t SdioCard::errorLine() const {
   return m_errorLine;
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 bool SdioCard::isBusy() {
   return m_busyFcn ? m_busyFcn() : m_initDone && isBusyCMD13();
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 uint32_t SdioCard::kHzSdClk() {
   return m_sdClkKhz;
 }
-//-----------------------------------------------------------------------------
-bool SdioCard::readBlock(uint32_t lba, uint8_t* buf) {
-  uint8_t aligned[512];
-
-  uint8_t* ptr = (uint32_t)buf & 3 ? aligned : buf;
-
-  if (!rdWrBlocks(CMD17_DMA_XFERTYP, lba, ptr, 1)) {
-    return sdError(SD_CARD_ERROR_CMD18);
-  }
-  if (ptr != buf) {
-    memcpy(buf, aligned, 512);
-  }
-  return true;
-}
-//-----------------------------------------------------------------------------
-bool SdioCard::readBlocks(uint32_t lba, uint8_t* buf, size_t n) {
-  if ((uint32_t)buf & 3) {
-    for (size_t i = 0; i < n; i++, lba++, buf += 512) {
-      if (!readBlock(lba, buf)) {
-        return false;  // readBlock will set errorCode.
-      }
-    }
-    return true;
-  }
-  if (!rdWrBlocks(CMD18_DMA_XFERTYP, lba, buf, n)) {
-    return sdError(SD_CARD_ERROR_CMD18);
-  }
-  return true;
-}
-//-----------------------------------------------------------------------------
-bool SdioCard::readCID(void* cid) {
+//------------------------------------------------------------------------------
+bool SdioCard::readCID(cid_t* cid) {
   memcpy(cid, &m_cid, 16);
   return true;
 }
-//-----------------------------------------------------------------------------
-bool SdioCard::readCSD(void* csd) {
+//------------------------------------------------------------------------------
+bool SdioCard::readCSD(csd_t* csd) {
   memcpy(csd, &m_csd, 16);
   return true;
 }
-//-----------------------------------------------------------------------------
-bool SdioCard::readData(uint8_t *dst) {
+//------------------------------------------------------------------------------
+bool SdioCard::readData(uint8_t* dst) {
   DBG_IRQSTAT();
-  uint32_t *p32 = reinterpret_cast<uint32_t*>(dst);
+  uint32_t* p32 = reinterpret_cast<uint32_t*>(dst);
 
   if (!(SDHC_PRSSTAT & SDHC_PRSSTAT_RTA)) {
     SDHC_PROCTL &= ~SDHC_PROCTL_SABGREQ;
-    if ((SDHC_BLKATTR & 0XFFFF0000) == 0X10000) {
-    // Don't stop at block gap if last block.  Allows auto CMD12.
-      SDHC_PROCTL |= SDHC_PROCTL_CREQ;
-    } else {
-      noInterrupts();
-      SDHC_PROCTL |= SDHC_PROCTL_CREQ;
-      SDHC_PROCTL |= SDHC_PROCTL_SABGREQ;
-      interrupts();
-    }
+    noInterrupts();
+    SDHC_PROCTL |= SDHC_PROCTL_CREQ;
+    SDHC_PROCTL |= SDHC_PROCTL_SABGREQ;
+    interrupts();
   }
   if (waitTimeout(isBusyFifoRead)) {
     return sdError(SD_CARD_ERROR_READ_FIFO);
@@ -665,94 +767,134 @@ bool SdioCard::readData(uint8_t *dst) {
   SDHC_IRQSTAT = m_irqstat;
   return (m_irqstat & SDHC_IRQSTAT_TC) && !(m_irqstat & SDHC_IRQSTAT_ERROR);
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 bool SdioCard::readOCR(uint32_t* ocr) {
   *ocr = m_ocr;
   return true;
 }
-//-----------------------------------------------------------------------------
-bool SdioCard::readStart(uint32_t lba) {
-  // K66/K65 Errata - SDHC: Does not support Infinite Block Transfer Mode.
-  return sdError(SD_CARD_ERROR_FUNCTION_NOT_SUPPORTED);
-}
-//-----------------------------------------------------------------------------
-// SDHC will do Auto CMD12 after count blocks.
-bool SdioCard::readStart(uint32_t lba, uint32_t count) {
-  DBG_IRQSTAT();
-  if (count > 0XFFFF) {
-    return sdError(SD_CARD_ERROR_READ_START);
+//------------------------------------------------------------------------------
+bool SdioCard::readSector(uint32_t sector, uint8_t* dst) {
+  if (m_sdioConfig.useDma()) {
+    uint8_t aligned[512];
+
+    uint8_t* ptr = (uint32_t)dst & 3 ? aligned : dst;
+
+    if (!rdWrSectors(CMD17_DMA_XFERTYP, sector, ptr, 1)) {
+      return sdError(SD_CARD_ERROR_CMD17);
+    }
+    if (ptr != dst) {
+      memcpy(dst, aligned, 512);
+    }
+  } else {
+    if (m_curState != READ_STATE || sector != m_curSector) {
+      if (!syncDevice()) {
+        return false;
+      }
+      if (!readStart(sector)) {
+        return false;
+      }
+      m_curSector = sector;
+      m_curState = READ_STATE;
+    }
+    if (!readData(dst)) {
+      return false;
+    }
+#if defined(__MK64FX512__) || defined(__MK66FX1M0__)
+    if ((SDHC_BLKATTR & 0XFFFF0000) == 0) {
+      if (!syncDevice()) {
+        return false;
+      }
+    }
+#endif  // defined(__MK64FX512__) || defined(__MK66FX1M0__)
+    m_curSector++;
   }
+  return true;
+}
+//------------------------------------------------------------------------------
+bool SdioCard::readSectors(uint32_t sector, uint8_t* dst, size_t n) {
+  if (m_sdioConfig.useDma()) {
+    if ((uint32_t)dst & 3) {
+      for (size_t i = 0; i < n; i++, sector++, dst += 512) {
+        if (!readSector(sector, dst)) {
+          return false;  // readSector will set errorCode.
+        }
+      }
+      return true;
+    }
+    if (!rdWrSectors(CMD18_DMA_XFERTYP, sector, dst, n)) {
+      return sdError(SD_CARD_ERROR_CMD18);
+    }
+  } else {
+    for (size_t i = 0; i < n; i++) {
+      if (!readSector(sector + i, dst + i*512UL)) {
+        return false;
+      }
+    }
+  }
+  return true;
+}
+//------------------------------------------------------------------------------
+// SDHC will do Auto CMD12 after count sectors.
+bool SdioCard::readStart(uint32_t sector) {
+  DBG_IRQSTAT();
   if (yieldTimeout(isBusyCMD13)) {
     return sdError(SD_CARD_ERROR_CMD13);
   }
-  if (count > 1) {
-    SDHC_PROCTL |= SDHC_PROCTL_SABGREQ;
-  }
-  SDHC_BLKATTR = SDHC_BLKATTR_BLKCNT(count) | SDHC_BLKATTR_BLKSIZE(512);
-  if (!cardCommand(CMD18_PGM_XFERTYP, m_highCapacity ? lba : 512*lba)) {
+  SDHC_PROCTL |= SDHC_PROCTL_SABGREQ;
+#if defined(__IMXRT1062__)
+  // Infinite transfer.
+  SDHC_BLKATTR = SDHC_BLKATTR_BLKSIZE(512);
+#else  // defined(__IMXRT1062__)
+  // Errata - can't do infinite transfer.
+  SDHC_BLKATTR = SDHC_BLKATTR_BLKCNT(0XFFFF) | SDHC_BLKATTR_BLKSIZE(512);
+#endif  // defined(__IMXRT1062__)
+
+  if (!cardCommand(CMD18_PGM_XFERTYP, m_highCapacity ? sector : 512*sector)) {
     return sdError(SD_CARD_ERROR_CMD18);
   }
   return true;
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 bool SdioCard::readStop() {
   return transferStop();
 }
-//-----------------------------------------------------------------------------
-bool SdioCard::syncBlocks() {
+//------------------------------------------------------------------------------
+uint32_t SdioCard::sectorCount() {
+  return sdCardCapacity(&m_csd);
+}
+//------------------------------------------------------------------------------
+uint32_t SdioCard::status() {
+  return statusCMD13();
+}
+//------------------------------------------------------------------------------
+bool SdioCard::syncDevice() {
+  if (m_curState == READ_STATE) {
+    m_curState = IDLE_STATE;
+    if (!readStop()) {
+      return false;
+    }
+  } else if (m_curState == WRITE_STATE) {
+    m_curState = IDLE_STATE;
+    if (!writeStop()) {
+      return false;
+    }
+  }
   return true;
 }
-//-----------------------------------------------------------------------------
-uint8_t SdioCard::type() {
+//------------------------------------------------------------------------------
+uint8_t SdioCard::type() const {
   return  m_version2 ? m_highCapacity ?
           SD_CARD_TYPE_SDHC : SD_CARD_TYPE_SD2 : SD_CARD_TYPE_SD1;
 }
-//-----------------------------------------------------------------------------
-bool SdioCard::writeBlock(uint32_t lba, const uint8_t* buf) {
-  uint8_t *ptr;
-  uint8_t aligned[512];
-  if (3 & (uint32_t)buf) {
-    ptr = aligned;
-    memcpy(aligned, buf, 512);
-  } else {
-    ptr = const_cast<uint8_t*>(buf);
-  }
-  if (!rdWrBlocks(CMD24_DMA_XFERTYP, lba, ptr, 1)) {
-    return sdError(SD_CARD_ERROR_CMD24);
-  }
-  return true;
-}
-//-----------------------------------------------------------------------------
-bool SdioCard::writeBlocks(uint32_t lba, const uint8_t* buf, size_t n) {
-  uint8_t* ptr = const_cast<uint8_t*>(buf);
-  if (3 & (uint32_t)ptr) {
-    for (size_t i = 0; i < n; i++, lba++, ptr += 512) {
-      if (!writeBlock(lba, ptr)) {
-        return false;  // writeBlock will set errorCode.
-      }
-    }
-    return true;
-  }
-  if (!rdWrBlocks(CMD25_DMA_XFERTYP, lba, ptr, n)) {
-    return sdError(SD_CARD_ERROR_CMD25);
-  }
-  return true;
-}
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 bool SdioCard::writeData(const uint8_t* src) {
   DBG_IRQSTAT();
   const uint32_t* p32 = reinterpret_cast<const uint32_t*>(src);
-
   if (!(SDHC_PRSSTAT & SDHC_PRSSTAT_WTA)) {
     SDHC_PROCTL &= ~SDHC_PROCTL_SABGREQ;
-    // Don't stop at block gap if last block.  Allows auto CMD12.
-    if ((SDHC_BLKATTR & 0XFFFF0000) == 0X10000) {
-      SDHC_PROCTL |= SDHC_PROCTL_CREQ;
-    } else {
-      SDHC_PROCTL |= SDHC_PROCTL_CREQ;
-      SDHC_PROCTL |= SDHC_PROCTL_SABGREQ;
-    }
+    SDHC_PROCTL |= SDHC_PROCTL_CREQ;
   }
+  SDHC_PROCTL |= SDHC_PROCTL_SABGREQ;
   if (waitTimeout(isBusyFifoWrite)) {
     return sdError(SD_CARD_ERROR_WRITE_FIFO);
   }
@@ -771,36 +913,96 @@ bool SdioCard::writeData(const uint8_t* src) {
   SDHC_IRQSTAT = m_irqstat;
   return (m_irqstat & SDHC_IRQSTAT_TC) && !(m_irqstat & SDHC_IRQSTAT_ERROR);
 }
-//-----------------------------------------------------------------------------
-bool SdioCard::writeStart(uint32_t lba) {
-  // K66/K65 Errata - SDHC: Does not support Infinite Block Transfer Mode.
-  return sdError(SD_CARD_ERROR_FUNCTION_NOT_SUPPORTED);
-}
-//-----------------------------------------------------------------------------
-// SDHC will do Auto CMD12 after count blocks.
-bool SdioCard::writeStart(uint32_t lba, uint32_t count) {
-  if (count > 0XFFFF) {
-    return sdError(SD_CARD_ERROR_WRITE_START);
+//------------------------------------------------------------------------------
+bool SdioCard::writeSector(uint32_t sector, const uint8_t* src) {
+  if (m_sdioConfig.useDma()) {
+    uint8_t* ptr;
+    uint8_t aligned[512];
+    if (3 & (uint32_t)src) {
+      ptr = aligned;
+      memcpy(aligned, src, 512);
+    } else {
+      ptr = const_cast<uint8_t*>(src);
+    }
+  if (!rdWrSectors(CMD24_DMA_XFERTYP, sector, ptr, 1)) {
+      return sdError(SD_CARD_ERROR_CMD24);
+    }
+  } else {
+    if (m_curState != WRITE_STATE || m_curSector != sector) {
+      if (!syncDevice()) {
+        return false;
+      }
+      if (!writeStart(sector )) {
+        return false;
+      }
+      m_curSector = sector;
+      m_curState = WRITE_STATE;
+    }
+    if (!writeData(src)) {
+      return false;
+    }
+    m_curSector++;
+#if defined(__MK64FX512__) || defined(__MK66FX1M0__)
+    // End transfer with CMD12 if required.
+    if ((SDHC_BLKATTR & 0XFFFF0000) == 0) {
+      if (!syncDevice()) {
+        return false;
+      }
+    }
+#endif  // defined(__MK64FX512__) || defined(__MK66FX1M0__)
   }
-  DBG_IRQSTAT();
+  return true;
+}
+//------------------------------------------------------------------------------
+bool SdioCard::writeSectors(uint32_t sector, const uint8_t* src, size_t n) {
+  if (m_sdioConfig.useDma()) {
+    uint8_t* ptr = const_cast<uint8_t*>(src);
+    if (3 & (uint32_t)ptr) {
+      for (size_t i = 0; i < n; i++, sector++, ptr += 512) {
+        if (!writeSector(sector, ptr)) {
+          return false;  // writeSector will set errorCode.
+        }
+      }
+      return true;
+    }
+    if (!rdWrSectors(CMD25_DMA_XFERTYP, sector, ptr, n)) {
+      return sdError(SD_CARD_ERROR_CMD25);
+    }
+  } else {
+    for (size_t i = 0; i < n; i++) {
+      if (!writeSector(sector + i, src + i*512UL)) {
+        return false;
+      }
+    }
+  }
+  return true;
+}
+//------------------------------------------------------------------------------
+bool SdioCard::writeStart(uint32_t sector) {
   if (yieldTimeout(isBusyCMD13)) {
     return sdError(SD_CARD_ERROR_CMD13);
   }
-  if (count > 1) {
-    SDHC_PROCTL |= SDHC_PROCTL_SABGREQ;
-  }
-  SDHC_BLKATTR = SDHC_BLKATTR_BLKCNT(count) | SDHC_BLKATTR_BLKSIZE(512);
+  SDHC_PROCTL &= ~SDHC_PROCTL_SABGREQ;
 
-  if (!cardCommand(CMD25_PGM_XFERTYP, m_highCapacity ? lba : 512*lba)) {
+#if defined(__IMXRT1062__)
+  // Infinite transfer.
+  SDHC_BLKATTR = SDHC_BLKATTR_BLKSIZE(512);
+#else  // defined(__IMXRT1062__)
+  // Errata - can't do infinite transfer.
+  SDHC_BLKATTR = SDHC_BLKATTR_BLKCNT(0XFFFF) | SDHC_BLKATTR_BLKSIZE(512);
+#endif  // defined(__IMXRT1062__)
+  if (!cardCommand(CMD25_PGM_XFERTYP, m_highCapacity ? sector : 512*sector)) {
     return sdError(SD_CARD_ERROR_CMD25);
   }
   return true;
 }
-//-----------------------------------------------------------------------------
+//------------------------------------------------------------------------------
 bool SdioCard::writeStop() {
   return transferStop();
 }
 
+
 }; // namespace sdfat
 
-#endif  // defined(__MK64FX512__) || defined(__MK66FX1M0__)
+
+#endif  // defined(__MK64FX512__)  defined(__MK66FX1M0__) defined(__IMXRT1062__)
