@@ -26,17 +26,32 @@
 
 #include <functional>
 #include <memory>
+#include <functional>
 #include <ESP8266WiFi.h>
 #include <FS.h>
 #include "detail/mimetable.h"
 #include "Uri.h"
 
+//#define DEBUG_ESP_HTTP_SERVER
+
+#ifdef DEBUG_ESP_HTTP_SERVER
+#ifdef DEBUG_ESP_PORT
+#define DBGWS(f,...) do { DEBUG_ESP_PORT.printf(PSTR(f), ##__VA_ARGS__); } while (0)
+#else
+#define DBGWS(f,...) do { Serial.printf(PSTR(f), ##__VA_ARGS__); } while (0)
+#endif
+#else
+#define DBGWS(x...) do { (void)0; } while (0)
+#endif
+
 enum HTTPMethod { HTTP_ANY, HTTP_GET, HTTP_HEAD, HTTP_POST, HTTP_PUT, HTTP_PATCH, HTTP_DELETE, HTTP_OPTIONS };
 enum HTTPUploadStatus { UPLOAD_FILE_START, UPLOAD_FILE_WRITE, UPLOAD_FILE_END,
                         UPLOAD_FILE_ABORTED };
 enum HTTPClientStatus { HC_NONE, HC_WAIT_READ, HC_WAIT_CLOSE };
 enum HTTPAuthMethod { BASIC_AUTH, DIGEST_AUTH };
 
+#define WEBSERVER_HAS_HOOK 1
+
 #define HTTP_DOWNLOAD_UNIT_SIZE 1460
 
 #ifndef HTTP_UPLOAD_BUFLEN
@@ -67,8 +82,12 @@ namespace esp8266webserver {
 template<typename ServerType>
 class ESP8266WebServerTemplate;
 
+}
+
 #include "detail/RequestHandler.h"
 
+namespace esp8266webserver {
+
 template<typename ServerType>
 class ESP8266WebServerTemplate
 {
@@ -80,6 +99,9 @@ public:
   using ClientType = typename ServerType::ClientType;
   using RequestHandlerType = RequestHandler<ServerType>;
   using WebServerType = ESP8266WebServerTemplate<ServerType>;
+  enum ClientFuture { CLIENT_REQUEST_CAN_CONTINUE, CLIENT_REQUEST_IS_HANDLED, CLIENT_MUST_STOP, CLIENT_IS_GIVEN };
+  typedef String (*ContentTypeFunction) (const String&);
+  using HookFunction = std::function<ClientFuture(const String& method, const String& url, WiFiClient* client, ContentTypeFunction contentType)>;
 
   void begin();
   void begin(uint16_t port);
@@ -103,7 +125,7 @@ public:
 
   const String& uri() const { return _currentUri; }
   HTTPMethod method() const { return _currentMethod; }
-  ClientType client() { return _currentClient; }
+  ClientType& client() { return _currentClient; }
   HTTPUpload& upload() { return *_currentUpload; }
 
   // Allows setting server options (i.e. SSL keys) by the instantiator
@@ -116,6 +138,8 @@ public:
   int args() const;                        // get arguments count
   bool hasArg(const String& name) const;   // check if argument exists
   void collectHeaders(const char* headerKeys[], const size_t headerKeysCount); // set the request headers to collect
+  template<typename... Args>
+  void collectHeaders(const Args&... args); // set the request headers to collect (variadic template version)
   const String& header(const String& name) const; // get request header value by name
   const String& header(int i) const;       // get request header value by number
   const String& headerName(int i) const;   // get request header name by number
@@ -127,7 +151,7 @@ public:
   // code - HTTP response code, can be 200 or 404
   // content_type - HTTP content type, like "text/plain" or "image/png"
   // content - actual content body
-  void send(int code, const char* content_type = NULL, const String& content = String(""));
+  void send(int code, const char* content_type = NULL, const String& content = emptyString);
   void send(int code, char* content_type, const String& content);
   void send(int code, const String& content_type, const String& content);
   void send(int code, const char *content_type, const char *content) {
@@ -142,14 +166,23 @@ public:
   void send_P(int code, PGM_P content_type, PGM_P content);
   void send_P(int code, PGM_P content_type, PGM_P content, size_t contentLength);
 
+  void send(int code, const char* content_type, Stream* stream, size_t content_length = 0);
+  void send(int code, const char* content_type, Stream& stream, size_t content_length = 0);
+
   void setContentLength(const size_t contentLength);
   void sendHeader(const String& name, const String& value, bool first = false);
   void sendContent(const String& content);
+  void sendContent(String& content) {
+    sendContent((const String&)content);
+  }
   void sendContent_P(PGM_P content);
   void sendContent_P(PGM_P content, size_t size);
   void sendContent(const char *content) { sendContent_P(content); }
   void sendContent(const char *content, size_t size) { sendContent_P(content, size); }
 
+  void sendContent(Stream* content, ssize_t content_length = 0);
+  void sendContent(Stream& content, ssize_t content_length = 0) { sendContent(&content, content_length); }
+
   bool chunkedResponseModeStart_P (int code, PGM_P content_type) {
     if (_currentVersion == 0)
         // no chunk mode in HTTP/1.0
@@ -168,6 +201,14 @@ public:
     sendContent(emptyString);
   }
 
+  // Whether other requests should be accepted from the client on the
+  // same socket after a response is sent.
+  // This will automatically configure the "Connection" header of the response.
+  // Defaults to true when the client's HTTP version is 1.1 or above, otherwise it defaults to false.
+  // If the client sends the "Connection" header, the value given by the header is used.
+  void keepAlive(bool keepAlive) { _keepAlive = keepAlive; }
+  bool keepAlive() { return _keepAlive; }
+
   static String credentialHash(const String& username, const String& realm, const String& password);
 
   static String urlDecode(const String& text);
@@ -185,24 +226,62 @@ public:
     size_t contentLength = 0;
     _streamFileCore(file.size(), file.name(), contentType);
     if (requestMethod == HTTP_GET) {
-      contentLength = _currentClient.write(file);
+      contentLength = file.sendAll(_currentClient);
     }
     return contentLength;
   }
 
+  // Implement GET and HEAD requests for stream
+  // Stream body on HTTP_GET but not on HTTP_HEAD requests.
+  template<typename T>
+  size_t stream(T &aStream, const String& contentType, HTTPMethod requestMethod, ssize_t size) {
+    setContentLength(size);
+    send(200, contentType, emptyString);
+    if (requestMethod == HTTP_GET)
+        size = aStream.sendSize(_currentClient, size);
+    return size;
+  }
+
+  // Implement GET and HEAD requests for stream
+  // Stream body on HTTP_GET but not on HTTP_HEAD requests.
+  template<typename T>
+  size_t stream(T& aStream, const String& contentType, HTTPMethod requestMethod = HTTP_GET) {
+    ssize_t size = aStream.size();
+    if (size < 0)
+    {
+        send(500, F("text/html"), F("input stream: undetermined size"));
+        return 0;
+    }
+    return stream(aStream, contentType, requestMethod, size);
+  }
+
   static String responseCodeToString(const int code);
 
+  void addHook (HookFunction hook) {
+    if (_hook) {
+      auto previousHook = _hook;
+      _hook = [previousHook, hook](const String& method, const String& url, WiFiClient* client, ContentTypeFunction contentType) {
+          auto whatNow = previousHook(method, url, client, contentType);
+          if (whatNow == CLIENT_REQUEST_CAN_CONTINUE)
+            return hook(method, url, client, contentType);
+          return whatNow;
+        };
+    } else {
+      _hook = hook;
+    }
+  }
+
 protected:
   void _addRequestHandler(RequestHandlerType* handler);
   void _handleRequest();
   void _finalizeResponse();
-  bool _parseRequest(ClientType& client);
+  ClientFuture _parseRequest(ClientType& client);
   void _parseArguments(const String& data);
   int _parseArgumentsPrivate(const String& data, std::function<void(String&,String&,const String&,int,int,int,int)> handler);
   bool _parseForm(ClientType& client, const String& boundary, uint32_t len);
   bool _parseFormUploadAborted();
   void _uploadWriteByte(uint8_t b);
-  uint8_t _uploadReadByte(ClientType& client);
+  int _uploadReadByte(ClientType& client);
   void _prepareHeader(String& response, int code, const char* content_type, size_t contentLength);
   bool _collectHeader(const char* headerName, const char* headerValue);
 
@@ -219,51 +298,49 @@ protected:
 
   ServerType  _server;
   ClientType  _currentClient;
-  HTTPMethod  _currentMethod;
+  HTTPMethod  _currentMethod = HTTP_ANY;
   String      _currentUri;
-  uint8_t     _currentVersion;
-  HTTPClientStatus _currentStatus;
-  unsigned long _statusChange;
+  uint8_t     _currentVersion = 0;
+  HTTPClientStatus _currentStatus = HC_NONE;
+  unsigned long _statusChange = 0;
 
-  RequestHandlerType*  _currentHandler;
-  RequestHandlerType*  _firstHandler;
-  RequestHandlerType*  _lastHandler;
+  RequestHandlerType*  _currentHandler = nullptr;
+  RequestHandlerType*  _firstHandler = nullptr;
+  RequestHandlerType*  _lastHandler = nullptr;
   THandlerFunction _notFoundHandler;
   THandlerFunction _fileUploadHandler;
 
-  int              _currentArgCount;
-  RequestArgument* _currentArgs;
+  int              _currentArgCount = 0;
+  RequestArgument* _currentArgs = nullptr;
+  int              _currentArgsHavePlain = 0;
   std::unique_ptr<HTTPUpload> _currentUpload;
-  int              _postArgsLen;
-  RequestArgument* _postArgs;
+  int              _postArgsLen = 0;
+  RequestArgument* _postArgs = nullptr;
 
-  int              _headerKeysCount;
-  RequestArgument* _currentHeaders;
+  int              _headerKeysCount = 0;
+  RequestArgument* _currentHeaders = nullptr;
 
-  size_t           _contentLength;
+  size_t           _contentLength = 0;
   String           _responseHeaders;
 
   String           _hostHeader;
-  bool             _chunked;
-  bool             _corsEnabled;
+  bool             _chunked = false;
+  bool             _corsEnabled = false;
+  bool             _keepAlive = false;
 
   String           _snonce;  // Store noance and opaque for future comparison
   String           _sopaque;
   String           _srealm;  // Store the Auth realm between Calls
 
-  
-
+  HookFunction     _hook;
 };
 
+} // namespace
 
 #include "ESP8266WebServer-impl.h"
 #include "Parsing-impl.h"
 
-};
-
-
 using ESP8266WebServer = esp8266webserver::ESP8266WebServerTemplate<WiFiServer>;
 using RequestHandler = esp8266webserver::RequestHandler<WiFiServer>;
 
-
-#endif //ESP8266WEBSERVER_H
+#endif //ESP8266WEBSERVER_H
\ No newline at end of file
