@@ -33,11 +33,11 @@ extern "C" {
 #include "osapi.h"
 #include "mem.h"
 #include "user_interface.h"
+#include <lwip/init.h> // LWIP_VERSION_*
 }
 
 #include "debug.h"
-
-
+#include "LwipDhcpServer.h"
 
 // -----------------------------------------------------------------------------------------------------------------------
 // ---------------------------------------------------- Private functions ------------------------------------------------
@@ -54,10 +54,13 @@ static bool softap_config_equal(const softap_config& lhs, const softap_config& r
  * @return equal
  */
 static bool softap_config_equal(const softap_config& lhs, const softap_config& rhs) {
-    if(strcmp(reinterpret_cast<const char*>(lhs.ssid), reinterpret_cast<const char*>(rhs.ssid)) != 0) {
+    if(lhs.ssid_len != rhs.ssid_len) {
         return false;
     }
-    if(strcmp(reinterpret_cast<const char*>(lhs.password), reinterpret_cast<const char*>(rhs.password)) != 0) {
+    if(memcmp(lhs.ssid, rhs.ssid, lhs.ssid_len) != 0) {
+        return false;
+    }
+    if(strncmp(reinterpret_cast<const char*>(lhs.password), reinterpret_cast<const char*>(rhs.password), sizeof(softap_config::password)) != 0) {
         return false;
     }
     if(lhs.channel != rhs.channel) {
@@ -85,13 +88,13 @@ static bool softap_config_equal(const softap_config& lhs, const softap_config& r
 
 /**
  * Set up an access point
- * @param ssid              Pointer to the SSID (max 31 char).
- * @param passphrase        For WPA2 min 8 char, for open use NULL (max 63 char).
+ * @param ssid              Pointer to the SSID (max 32 char).
+ * @param psk               For WPA2 min 8 char max 64 char, for open use "" or NULL.
  * @param channel           WiFi channel number, 1 - 13.
  * @param ssid_hidden       Network cloaking (0 = broadcast SSID, 1 = hide SSID)
  * @param max_connection    Max simultaneous connected clients, 0 - 8. https://bbs.espressif.com/viewtopic.php?f=46&t=481&p=1832&hilit=max_connection#p1832
  */
-bool ESP8266WiFiAPClass::softAP(const char* ssid, const char* passphrase, int channel, int ssid_hidden, int max_connection) {
+bool ESP8266WiFiAPClass::softAP(const char* ssid, const char* psk, int channel, int ssid_hidden, int max_connection) {
 
     if(!WiFi.enableAP(true)) {
         // enable AP failed
@@ -99,36 +102,43 @@ bool ESP8266WiFiAPClass::softAP(const char* ssid, const char* passphrase, int ch
         return false;
     }
 
-    if(!ssid || strlen(ssid) == 0 || strlen(ssid) > 31) {
-        // fail SSID too long or missing!
-        DEBUG_WIFI("[AP] SSID too long or missing!\n");
+    size_t ssid_len = ssid ? strlen(ssid) : 0;
+    if(ssid_len == 0 || ssid_len > 32) {
+        DEBUG_WIFI("[AP] SSID length %u, too long or missing!\n", ssid_len);
         return false;
     }
 
-    if(passphrase && strlen(passphrase) > 0 && (strlen(passphrase) > 63 || strlen(passphrase) < 8)) {
-        // fail passphrase to long or short!
-        DEBUG_WIFI("[AP] fail passphrase too long or short!\n");
+    size_t psk_len = psk ? strlen(psk) : 0;
+    if(psk_len > 0 && (psk_len > 64 || psk_len < 8)) {
+        DEBUG_WIFI("[AP] fail psk length %u, too long or short!\n", psk_len);
         return false;
     }
 
     bool ret = true;
 
     struct softap_config conf;
-    strcpy(reinterpret_cast<char*>(conf.ssid), ssid);
+    memcpy(reinterpret_cast<char*>(conf.ssid), ssid, ssid_len);
+    if (ssid_len < sizeof(conf.ssid)) {
+        conf.ssid[ssid_len] = 0;
+    }
+    conf.ssid_len = ssid_len;
+
+    if(psk_len) {
+        conf.authmode = AUTH_WPA2_PSK;
+        memcpy(reinterpret_cast<char*>(conf.password), psk, psk_len);
+        if (psk_len < sizeof(conf.password)) {
+            conf.password[psk_len] = 0;
+        }
+    } else {
+        conf.authmode = AUTH_OPEN;
+        conf.password[0] = 0;
+    }
+
     conf.channel = channel;
-    conf.ssid_len = strlen(ssid);
     conf.ssid_hidden = ssid_hidden;
     conf.max_connection = max_connection;
     conf.beacon_interval = 100;
 
-    if(!passphrase || strlen(passphrase) == 0) {
-        conf.authmode = AUTH_OPEN;
-        *conf.password = 0;
-    } else {
-        conf.authmode = AUTH_WPA2_PSK;
-        strcpy(reinterpret_cast<char*>(conf.password), passphrase);
-    }
-
     struct softap_config conf_compare;
     if(WiFi._persistent){
         wifi_softap_get_config_default(&conf_compare);
@@ -156,13 +166,7 @@ bool ESP8266WiFiAPClass::softAP(const char* ssid, const char* passphrase, int ch
         DEBUG_WIFI("[AP] softap config unchanged\n");
     }
 
-    if(wifi_softap_dhcps_status() != DHCP_STARTED) {
-        DEBUG_WIFI("[AP] DHCP not started, starting...\n");
-        if(!wifi_softap_dhcps_start()) {
-            DEBUG_WIFI("[AP] wifi_softap_dhcps_start failed!\n");
-            ret = false;
-        }
-    }
+    dhcpSoftAP.end();
 
     // check IP config
     struct ip_info ip;
@@ -182,11 +186,13 @@ bool ESP8266WiFiAPClass::softAP(const char* ssid, const char* passphrase, int ch
         ret = false;
     }
 
+    dhcpSoftAP.begin(&ip);
+
     return ret;
 }
 
-bool ESP8266WiFiAPClass::softAP(const String& ssid, const String& passphrase, int channel, int ssid_hidden, int max_connection) {
-    return softAP(ssid.c_str(), passphrase.c_str(), channel, ssid_hidden, max_connection);
+bool ESP8266WiFiAPClass::softAP(const String& ssid, const String& psk, int channel, int ssid_hidden, int max_connection) {
+    return softAP(ssid.c_str(), psk.c_str(), channel, ssid_hidden, max_connection);
 }
 
 /**
@@ -228,6 +234,7 @@ bool ESP8266WiFiAPClass::softAPConfig(IPAddress local_ip, IPAddress gateway, IPA
     }
 
     struct dhcps_lease dhcp_lease;
+    dhcp_lease.enable = true;
     IPAddress ip = local_ip;
     ip[3] += 99;
     dhcp_lease.start_ip.addr = ip.v4();
@@ -237,19 +244,22 @@ bool ESP8266WiFiAPClass::softAPConfig(IPAddress local_ip, IPAddress gateway, IPA
     dhcp_lease.end_ip.addr = ip.v4();
     DEBUG_WIFI("[APConfig] DHCP IP end: %s\n", ip.toString().c_str());
 
-    if(!wifi_softap_set_dhcps_lease(&dhcp_lease)) {
+    if(!dhcpSoftAP.set_dhcps_lease(&dhcp_lease))
+    {
         DEBUG_WIFI("[APConfig] wifi_set_ip_info failed!\n");
         ret = false;
     }
 
     // set lease time to 720min --> 12h
-    if(!wifi_softap_set_dhcps_lease_time(720)) {
+    if(!dhcpSoftAP.set_dhcps_lease_time(720))
+    {
         DEBUG_WIFI("[APConfig] wifi_softap_set_dhcps_lease_time failed!\n");
         ret = false;
     }
 
     uint8 mode = info.gw.addr ? 1 : 0;
-    if(!wifi_softap_set_dhcps_offer_option(OFFER_ROUTER, &mode)) {
+    if(!dhcpSoftAP.set_dhcps_offer_option(OFFER_ROUTER, &mode))
+    {
         DEBUG_WIFI("[APConfig] wifi_softap_set_dhcps_offer_option failed!\n");
         ret = false;
     }
@@ -265,8 +275,7 @@ bool ESP8266WiFiAPClass::softAPConfig(IPAddress local_ip, IPAddress gateway, IPA
             DEBUG_WIFI("[APConfig] IP config Invalid?!\n");
             ret = false;
         } else if(local_ip.v4() != info.ip.addr) {
-            ip = info.ip.addr;
-            DEBUG_WIFI("[APConfig] IP config not set correct?! new IP: %s\n", ip.toString().c_str());
+            DEBUG_WIFI("[APConfig] IP config not set correct?! new IP: %s\n", IPAddress(info.ip.addr).toString().c_str());
             ret = false;
         }
     } else {
@@ -359,25 +368,24 @@ String ESP8266WiFiAPClass::softAPmacAddress(void) {
 String ESP8266WiFiAPClass::softAPSSID() const {
     struct softap_config config;
     wifi_softap_get_config(&config);
-    char* name = reinterpret_cast<char*>(config.ssid);
-    char ssid[sizeof(config.ssid) + 1];
-    memcpy(ssid, name, sizeof(config.ssid));
-    ssid[sizeof(config.ssid)] = '\0';
 
-    return String(ssid);
+    String ssid;
+    ssid.concat(reinterpret_cast<const char*>(config.ssid), config.ssid_len);
+
+    return ssid;
 }
 
 /**
- * Get the configured(Not-In-Flash) softAP PSK or PASSWORD.
+ * Get the configured(Not-In-Flash) softAP PSK.
  * @return String psk.
  */
 String ESP8266WiFiAPClass::softAPPSK() const {
     struct softap_config config;
     wifi_softap_get_config(&config);
-    char* pass = reinterpret_cast<char*>(config.password);
-    char psk[sizeof(config.password) + 1];
-    memcpy(psk, pass, sizeof(config.password));
-    psk[sizeof(config.password)] = '\0';
 
-    return String(psk);
+    char* ptr = reinterpret_cast<char*>(config.password);
+    String psk;
+    psk.concat(ptr, strnlen(ptr, sizeof(config.password)));
+
+    return psk;
 }
