@@ -1,10 +1,10 @@
 /**
  *
  * @file ESP8266WiFiMulti.h
- * @date 16.05.2015
- * @author Markus Sattler
+ * @date 30.09.2020
+ * @author Markus Sattler, Erriez
  *
- * Copyright (c) 2015 Markus Sattler. All rights reserved.
+ * Copyright (c) 2015-2020 Markus Sattler. All rights reserved.
  * This file is part of the esp8266 core for Arduino environment.
  *
  * This library is free software; you can redistribute it and/or
@@ -24,8 +24,8 @@
  */
 
 
-#ifndef WIFICLIENTMULTI_H_
-#define WIFICLIENTMULTI_H_
+#ifndef WIFI_CLIENT_MULTI_H_
+#define WIFI_CLIENT_MULTI_H_
 
 #include "ESP8266WiFi.h"
 #include <vector>
@@ -40,31 +40,47 @@
 #define DEBUG_WIFI_MULTI(...) do { (void)0; } while (0)
 #endif
 
+//! Default WiFi connection timeout in ms
+#ifndef WIFI_CONNECT_TIMEOUT_MS
+#define WIFI_CONNECT_TIMEOUT_MS     5000
+#endif
+
+//! Default WiFi scan timeout in ms
+#ifndef WIFI_SCAN_TIMEOUT_MS
+#define WIFI_SCAN_TIMEOUT_MS        5000
+#endif
+
 struct WifiAPEntry {
-    char * ssid;
-    char * passphrase;
+    char *ssid;
+    char *passphrase;
 };
 
 typedef std::vector<WifiAPEntry> WifiAPlist;
 
-class ESP8266WiFiMulti {
-    public:
-        ESP8266WiFiMulti();
-        ~ESP8266WiFiMulti();
+class ESP8266WiFiMulti
+{
+public:
+    ESP8266WiFiMulti();
+    ~ESP8266WiFiMulti();
 
-        bool addAP(const char* ssid, const char *passphrase = NULL);
-        bool existsAP(const char* ssid, const char *passphrase = NULL);
+    bool addAP(const char *ssid, const char *passphrase = NULL);
+    bool existsAP(const char *ssid, const char *passphrase = NULL);
 
-        wl_status_t run(uint32_t connectTimeoutMs=5000);
+    wl_status_t run(uint32_t connectTimeoutMs=WIFI_CONNECT_TIMEOUT_MS);
 
-        void cleanAPlist(void);
+    void cleanAPlist();
 
-    private:
-        WifiAPlist APlist;
-        bool APlistAdd(const char* ssid, const char *passphrase = NULL);
-        bool APlistExists(const char* ssid, const char *passphrase = NULL);
-        void APlistClean(void);
+private:
+    WifiAPlist _APlist;
+    bool _firstRun;
 
+    bool APlistAdd(const char *ssid, const char *passphrase = NULL);
+    bool APlistExists(const char *ssid, const char *passphrase = NULL);
+    void APlistClean();
+
+    wl_status_t connectWiFiMulti(uint32_t connectTimeoutMs);
+    int8_t startScan();
+    void printWiFiScan();
 };
 
-#endif /* WIFICLIENTMULTI_H_ */
+#endif // WIFI_CLIENT_MULTI_H_
