@@ -26,6 +26,7 @@
 #include "ESP8266WiFiGeneric.h"
 #include "ESP8266WiFiSTA.h"
 #include "PolledTimeout.h"
+#include "LwipIntf.h"
 
 #include "c_types.h"
 #include "ets_sys.h"
@@ -39,10 +40,6 @@ extern "C" {
 #include "lwip/err.h"
 #include "lwip/dns.h"
 #include "lwip/dhcp.h"
-#include "lwip/init.h" // LWIP_VERSION_
-#if LWIP_IPV6
-#include "lwip/netif.h" // struct netif
-#endif
 }
 
 #include "debug.h"
@@ -249,7 +246,6 @@ wl_status_t ESP8266WiFiSTAClass::begin() {
  * @param dns2       Static DNS server 2
  */
 
-#if LWIP_VERSION_MAJOR != 1
 /*
   About the following call in the end of ESP8266WiFiSTAClass::config():
       netif_set_addr(eagle_lwip_getif(STATION_IF), &info.ip, &info.netmask, &info.gw);
@@ -267,7 +263,6 @@ wl_status_t ESP8266WiFiSTAClass::begin() {
 #undef netif_set_addr // need to call lwIP-v1.4 netif_set_addr()
 extern "C" struct netif* eagle_lwip_getif (int netif_index);
 extern "C" void netif_set_addr (struct netif* netif, ip4_addr_t* ip, ip4_addr_t* netmask, ip4_addr_t* gw);
-#endif
 
 bool ESP8266WiFiSTAClass::config(IPAddress local_ip, IPAddress arg1, IPAddress arg2, IPAddress arg3, IPAddress dns2) {
 
@@ -287,30 +282,11 @@ bool ESP8266WiFiSTAClass::config(IPAddress local_ip, IPAddress arg1, IPAddress a
       return true;
   }
 
-  //To allow compatibility, check first octet of 3rd arg. If 255, interpret as ESP order, otherwise Arduino order.
-  IPAddress gateway = arg1;
-  IPAddress subnet = arg2;
-  IPAddress dns1 = arg3;
-
-  if(subnet[0] != 255)
-  {
-    //octet is not 255 => interpret as Arduino order
-    gateway = arg2;
-    subnet = arg3[0] == 0 ? IPAddress(255,255,255,0) : arg3; //arg order is arduino and 4th arg not given => assign it arduino default
-    dns1 = arg1;
-  }
-
-  // check whether all is IPv4 (or gateway not set)
-  if (!(local_ip.isV4() && subnet.isV4() && (!gateway.isSet() || gateway.isV4()))) {
+  IPAddress gateway, subnet, dns1;
+  if (!ipAddressReorder(local_ip, arg1, arg2, arg3, gateway, subnet, dns1))
     return false;
-  }
 
-  //ip and gateway must be in the same subnet
-  if((local_ip.v4() & subnet.v4()) != (gateway.v4() & subnet.v4())) {
-    return false;
-  }
-
-#if LWIP_VERSION_MAJOR != 1 && !CORE_MOCK
+#if !CORE_MOCK
   // get current->previous IP address
   // (check below)
   struct ip_info previp;
@@ -339,7 +315,7 @@ bool ESP8266WiFiSTAClass::config(IPAddress local_ip, IPAddress arg1, IPAddress a
       dns_setserver(1, dns2);
   }
 
-#if LWIP_VERSION_MAJOR != 1 && !CORE_MOCK
+#if !CORE_MOCK
   // trigger address change by calling lwIP-v1.4 api
   // (see explanation above)
   // only when ip is already set by other mean (generally dhcp)
@@ -455,10 +431,10 @@ int8_t ESP8266WiFiSTAClass::waitForConnectResult(unsigned long timeoutLength) {
     if((wifi_get_opmode() & 1) == 0) {
         return WL_DISCONNECTED;
     }
-    using esp8266::polledTimeout::oneShot;
-    oneShot timeout(timeoutLength); // number of milliseconds to wait before returning timeout error
+    // if probing doesn't trip, this yields
+    using oneShotYieldMs = esp8266::polledTimeout::timeoutTemplate<false, esp8266::polledTimeout::YieldPolicy::YieldOrSkip>;
+    oneShotYieldMs timeout(timeoutLength); // number of milliseconds to wait before returning timeout error
     while(!timeout) {
-        yield();
         if(status() != WL_DISCONNECTED) {
             return status();
         }
@@ -525,104 +501,19 @@ IPAddress ESP8266WiFiSTAClass::gatewayIP() {
  * @return IPAddress DNS Server IP
  */
 IPAddress ESP8266WiFiSTAClass::dnsIP(uint8_t dns_no) {
-#if LWIP_VERSION_MAJOR == 1
-    ip_addr_t dns_ip = dns_getserver(dns_no);
-    return IPAddress(dns_ip.addr);
-#else
     return IPAddress(dns_getserver(dns_no));
-#endif
-}
-
-
-/**
- * Get ESP8266 station DHCP hostname
- * @return hostname
- */
-String ESP8266WiFiSTAClass::hostname(void) {
-    return wifi_station_get_hostname();
 }
 
 /**
- * Set ESP8266 station DHCP hostname
- * @param aHostname max length:24
- * @return ok
+ * Get the broadcast ip address.
+ * @return IPAddress Broadcast IP
  */
-bool ESP8266WiFiSTAClass::hostname(const char* aHostname) {
-  /*
-  vvvv RFC952 vvvv
-  ASSUMPTIONS
-  1. A "name" (Net, Host, Gateway, or Domain name) is a text string up
-   to 24 characters drawn from the alphabet (A-Z), digits (0-9), minus
-   sign (-), and period (.).  Note that periods are only allowed when
-   they serve to delimit components of "domain style names". (See
-   RFC-921, "Domain Name System Implementation Schedule", for
-   background).  No blank or space characters are permitted as part of a
-   name. No distinction is made between upper and lower case.  The first
-   character must be an alpha character.  The last character must not be
-   a minus sign or period.  A host which serves as a GATEWAY should have
-   "-GATEWAY" or "-GW" as part of its name.  Hosts which do not serve as
-   Internet gateways should not use "-GATEWAY" and "-GW" as part of
-   their names. A host which is a TAC should have "-TAC" as the last
-   part of its host name, if it is a DoD host.  Single character names
-   or nicknames are not allowed.
-  ^^^^ RFC952 ^^^^
+IPAddress ESP8266WiFiSTAClass::broadcastIP()
+{
+    struct ip_info ip;
+    wifi_get_ip_info(STATION_IF, &ip);
 
-  - 24 chars max
-  - only a..z A..Z 0..9 '-'
-  - no '-' as last char
-  */
-
-    size_t len = strlen(aHostname);
-
-    if (len == 0 || len > 32) {
-        // nonos-sdk limit is 32
-        // (dhcp hostname option minimum size is ~60)
-        DEBUG_WIFI_GENERIC("WiFi.(set)hostname(): empty or large(>32) name\n");
-        return false;
-    }
-
-    // check RFC compliance
-    bool compliant = (len <= 24);
-    for (size_t i = 0; compliant && i < len; i++)
-        if (!isalnum(aHostname[i]) && aHostname[i] != '-')
-            compliant = false;
-    if (aHostname[len - 1] == '-')
-        compliant = false;
-
-    if (!compliant) {
-        DEBUG_WIFI_GENERIC("hostname '%s' is not compliant with RFC952\n", aHostname);
-    }
-
-    bool ret = wifi_station_set_hostname(aHostname);
-    if (!ret) {
-        DEBUG_WIFI_GENERIC("WiFi.hostname(%s): wifi_station_set_hostname() failed\n", aHostname);
-        return false;
-    }
-
-    // now we should inform dhcp server for this change, using lwip_renew()
-    // looping through all existing interface
-    // harmless for AP, also compatible with ethernet adapters (to come)
-    for (netif* intf = netif_list; intf; intf = intf->next) {
-
-        // unconditionally update all known interfaces
-#if LWIP_VERSION_MAJOR == 1
-        intf->hostname = (char*)wifi_station_get_hostname();
-#else
-        intf->hostname = wifi_station_get_hostname();
-#endif
-
-        if (netif_dhcp_data(intf) != nullptr) {
-            // renew already started DHCP leases
-            err_t lwipret = dhcp_renew(intf);
-            if (lwipret != ERR_OK) {
-                DEBUG_WIFI_GENERIC("WiFi.hostname(%s): lwIP error %d on interface %c%c (index %d)\n",
-                                   intf->hostname, (int)lwipret, intf->name[0], intf->name[1], intf->num);
-                ret = false;
-            }
-        }
-    }
-
-    return ret && compliant;
+    return IPAddress(ip.ip.addr | ~(ip.netmask.addr));
 }
 
 /**
@@ -639,8 +530,9 @@ wl_status_t ESP8266WiFiSTAClass::status() {
         case STATION_NO_AP_FOUND:
             return WL_NO_SSID_AVAIL;
         case STATION_CONNECT_FAIL:
-        case STATION_WRONG_PASSWORD:
             return WL_CONNECT_FAILED;
+        case STATION_WRONG_PASSWORD:
+            return WL_WRONG_PASSWORD;
         case STATION_IDLE:
             return WL_IDLE_STATUS;
         default:
@@ -700,7 +592,7 @@ String ESP8266WiFiSTAClass::BSSIDstr(void) {
  * Return the current network RSSI.
  * @return  RSSI value
  */
-int32_t ESP8266WiFiSTAClass::RSSI(void) {
+int8_t ESP8266WiFiSTAClass::RSSI(void) {
     return wifi_station_get_rssi();
 }
 
