@@ -30,12 +30,12 @@ extern "C" uint32_t _FS_start;
 extern "C" uint32_t _FS_end;
 
 ESP8266HTTPUpdate::ESP8266HTTPUpdate(void)
-        : _httpClientTimeout(8000), _followRedirects(HTTPC_DISABLE_FOLLOW_REDIRECTS), _ledPin(-1)
+        : _httpClientTimeout(8000)
 {
 }
 
 ESP8266HTTPUpdate::ESP8266HTTPUpdate(int httpClientTimeout)
-        : _httpClientTimeout(httpClientTimeout), _followRedirects(HTTPC_DISABLE_FOLLOW_REDIRECTS), _ledPin(-1)
+        : _httpClientTimeout(httpClientTimeout)
 {
 }
 
@@ -43,50 +43,26 @@ ESP8266HTTPUpdate::~ESP8266HTTPUpdate(void)
 {
 }
 
-#if HTTPUPDATE_1_2_COMPATIBLE
-HTTPUpdateResult ESP8266HTTPUpdate::update(const String& url, const String& currentVersion,
-        const String& httpsFingerprint, bool reboot)
+/**
+ * set the Authorization for the http request
+ * @param user const String&
+ * @param password const String&
+ */
+void ESP8266HTTPUpdate::setAuthorization(const String &user, const String &password)
 {
-    rebootOnUpdate(reboot);
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-    return update(url, currentVersion, httpsFingerprint);
-#pragma GCC diagnostic pop
+    _user = user;
+    _password = password;
 }
 
-HTTPUpdateResult ESP8266HTTPUpdate::update(const String& url, const String& currentVersion)
+/**
+ * set the Authorization for the http request
+ * @param auth const String& base64
+ */
+void ESP8266HTTPUpdate::setAuthorization(const String &auth)
 {
-    HTTPClient http;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-    http.begin(url);
-#pragma GCC diagnostic pop
-    return handleUpdate(http, currentVersion, false);
+    _auth = auth;
 }
 
-HTTPUpdateResult ESP8266HTTPUpdate::update(const String& url, const String& currentVersion,
-        const String& httpsFingerprint)
-{
-    HTTPClient http;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-    http.begin(url, httpsFingerprint);
-#pragma GCC diagnostic pop
-    return handleUpdate(http, currentVersion, false);
-}
-
-HTTPUpdateResult ESP8266HTTPUpdate::update(const String& url, const String& currentVersion,
-        const uint8_t httpsFingerprint[20])
-{
-    HTTPClient http;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-    http.begin(url, httpsFingerprint);
-#pragma GCC diagnostic pop
-    return handleUpdate(http, currentVersion, false);
-}
-#endif
-
 HTTPUpdateResult ESP8266HTTPUpdate::update(WiFiClient& client, const String& url, const String& currentVersion)
 {
     HTTPClient http;
@@ -94,38 +70,6 @@ HTTPUpdateResult ESP8266HTTPUpdate::update(WiFiClient& client, const String& url
     return handleUpdate(http, currentVersion, false);
 }
 
-#if HTTPUPDATE_1_2_COMPATIBLE
-HTTPUpdateResult ESP8266HTTPUpdate::updateSpiffs(const String& url, const String& currentVersion, const String& httpsFingerprint)
-{
-    HTTPClient http;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-    http.begin(url, httpsFingerprint);
-#pragma GCC diagnostic pop
-    return handleUpdate(http, currentVersion, true);
-}
-
-HTTPUpdateResult ESP8266HTTPUpdate::updateSpiffs(const String& url, const String& currentVersion, const uint8_t httpsFingerprint[20])
-{
-    HTTPClient http;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-    http.begin(url, httpsFingerprint);
-#pragma GCC diagnostic pop
-    return handleUpdate(http, currentVersion, true);
-}
-
-HTTPUpdateResult ESP8266HTTPUpdate::updateSpiffs(const String& url, const String& currentVersion)
-{
-    HTTPClient http;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-    http.begin(url);
-#pragma GCC diagnostic pop
-    return handleUpdate(http, currentVersion, true);
-}
-#endif
-
 HTTPUpdateResult ESP8266HTTPUpdate::updateFS(WiFiClient& client, const String& url, const String& currentVersion)
 {
     HTTPClient http;
@@ -133,56 +77,6 @@ HTTPUpdateResult ESP8266HTTPUpdate::updateFS(WiFiClient& client, const String& u
     return handleUpdate(http, currentVersion, true);
 }
 
-#if HTTPUPDATE_1_2_COMPATIBLE
-HTTPUpdateResult ESP8266HTTPUpdate::update(const String& host, uint16_t port, const String& uri, const String& currentVersion,
-        bool https, const String& httpsFingerprint, bool reboot)
-{
-    (void)https;
-    rebootOnUpdate(reboot);
-    if (httpsFingerprint.length() == 0) {
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-        return update(host, port, uri, currentVersion);
-    } else {
-        return update(host, port, uri, currentVersion, httpsFingerprint);
-#pragma GCC diagnostic pop
-    }
-}
-
-HTTPUpdateResult ESP8266HTTPUpdate::update(const String& host, uint16_t port, const String& uri,
-        const String& currentVersion)
-{
-    HTTPClient http;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-    http.begin(host, port, uri);
-#pragma GCC diagnostic pop
-    return handleUpdate(http, currentVersion, false);
-}
-
-HTTPUpdateResult ESP8266HTTPUpdate::update(const String& host, uint16_t port, const String& url,
-        const String& currentVersion, const String& httpsFingerprint)
-{
-    HTTPClient http;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-    http.begin(host, port, url, httpsFingerprint);
-#pragma GCC diagnostic pop
-    return handleUpdate(http, currentVersion, false);
-}
-
-HTTPUpdateResult ESP8266HTTPUpdate::update(const String& host, uint16_t port, const String& url,
-        const String& currentVersion, const uint8_t httpsFingerprint[20])
-{
-    HTTPClient http;
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored  "-Wdeprecated-declarations"
-    http.begin(host, port, url, httpsFingerprint);
-#pragma GCC diagnostic pop
-    return handleUpdate(http, currentVersion, false);
-}
-#endif
-
 HTTPUpdateResult ESP8266HTTPUpdate::update(WiFiClient& client, const String& host, uint16_t port, const String& uri,
         const String& currentVersion)
 {
@@ -241,6 +135,8 @@ String ESP8266HTTPUpdate::getLastErrorString(void)
         return F("Verify Bin Header Failed");
     case HTTP_UE_BIN_FOR_WRONG_FLASH:
         return F("New Binary Does Not Fit Flash Size");
+    case HTTP_UE_SERVER_UNAUTHORIZED:
+        return F("Unauthorized (401)");
     }
 
     return String();
@@ -282,6 +178,16 @@ HTTPUpdateResult ESP8266HTTPUpdate::handleUpdate(HTTPClient& http, const String&
         http.addHeader(F("x-ESP8266-version"), currentVersion);
     }
 
+    if (!_user.isEmpty() && !_password.isEmpty())
+    {
+        http.setAuthorization(_user.c_str(), _password.c_str());
+    }
+
+    if (!_auth.isEmpty())
+    {
+        http.setAuthorization(_auth.c_str());
+    }
+
     const char * headerkeys[] = { "x-MD5" };
     size_t headerkeyssize = sizeof(headerkeys) / sizeof(char*);
 
@@ -432,6 +338,10 @@ HTTPUpdateResult ESP8266HTTPUpdate::handleUpdate(HTTPClient& http, const String&
         _setLastError(HTTP_UE_SERVER_FORBIDDEN);
         ret = HTTP_UPDATE_FAILED;
         break;
+    case HTTP_CODE_UNAUTHORIZED:
+        _setLastError(HTTP_UE_SERVER_UNAUTHORIZED);
+        ret = HTTP_UPDATE_FAILED;
+        break;
     default:
         _setLastError(HTTP_UE_SERVER_WRONG_HTTP_CODE);
         ret = HTTP_UPDATE_FAILED;
