@@ -32,10 +32,6 @@
 #include <WiFiUdp.h>
 #include <ESP8266HTTPClient.h>
 
-#ifndef HTTPUPDATE_1_2_COMPATIBLE
-#define HTTPUPDATE_1_2_COMPATIBLE HTTPCLIENT_1_1_COMPATIBLE
-#endif
-
 #ifdef DEBUG_ESP_HTTP_UPDATE
 #ifdef DEBUG_ESP_PORT
 #define DEBUG_HTTP_UPDATE(fmt, ...) DEBUG_ESP_PORT.printf_P( (PGM_P)PSTR(fmt), ## __VA_ARGS__ )
@@ -56,6 +52,7 @@ constexpr int HTTP_UE_SERVER_WRONG_HTTP_CODE    = (-104);
 constexpr int HTTP_UE_SERVER_FAULTY_MD5         = (-105);
 constexpr int HTTP_UE_BIN_VERIFY_HEADER_FAILED  = (-106);
 constexpr int HTTP_UE_BIN_FOR_WRONG_FLASH       = (-107);
+constexpr int HTTP_UE_SERVER_UNAUTHORIZED       = (-108);
 
 enum HTTPUpdateResult {
     HTTP_UPDATE_FAILED,
@@ -92,7 +89,7 @@ public:
         _followRedirects = follow ? HTTPC_STRICT_FOLLOW_REDIRECTS : HTTPC_DISABLE_FOLLOW_REDIRECTS;
     }
     /**
-      * set redirect follow mode. See `followRedirects_t` enum for avaliable modes.
+      * set redirect follow mode. See `followRedirects_t` enum for available modes.
       * @param follow
       */
     void setFollowRedirects(followRedirects_t follow)
@@ -111,45 +108,13 @@ public:
         _ledOn = ledOn;
     }
 
-#if HTTPUPDATE_1_2_COMPATIBLE
-    // This function is deprecated, use rebootOnUpdate and the next one instead
-    t_httpUpdate_return update(const String& url, const String& currentVersion,
-                               const String& httpsFingerprint, bool reboot) __attribute__((deprecated));
-    t_httpUpdate_return update(const String& url, const String& currentVersion = "") __attribute__((deprecated));
-    t_httpUpdate_return update(const String& url, const String& currentVersion,
-                               const String& httpsFingerprint) __attribute__((deprecated));
-    t_httpUpdate_return update(const String& url, const String& currentVersion,
-                               const uint8_t httpsFingerprint[20]) __attribute__((deprecated)); // BearSSL
-#endif
+    void setAuthorization(const String& user, const String& password);
+    void setAuthorization(const String& auth);
+
     t_httpUpdate_return update(WiFiClient& client, const String& url, const String& currentVersion = "");
-
-#if HTTPUPDATE_1_2_COMPATIBLE
-    // This function is deprecated, use one of the overloads below along with rebootOnUpdate
-    t_httpUpdate_return update(const String& host, uint16_t port, const String& uri, const String& currentVersion,
-                               bool https, const String& httpsFingerprint, bool reboot) __attribute__((deprecated));
-
-    t_httpUpdate_return update(const String& host, uint16_t port, const String& uri = "/",
-                               const String& currentVersion = "") __attribute__((deprecated));
-    t_httpUpdate_return update(const String& host, uint16_t port, const String& url,
-                               const String& currentVersion, const String& httpsFingerprint) __attribute__((deprecated));
-    t_httpUpdate_return update(const String& host, uint16_t port, const String& url,
-                               const String& currentVersion, const uint8_t httpsFingerprint[20]) __attribute__((deprecated)); // BearSSL
-#endif
     t_httpUpdate_return update(WiFiClient& client, const String& host, uint16_t port, const String& uri = "/",
                                const String& currentVersion = "");
-
-#if HTTPUPDATE_1_2_COMPATIBLE
-    // This function is deprecated, use rebootOnUpdate and the next one instead
-    t_httpUpdate_return updateSpiffs(const String& url, const String& currentVersion,
-                                     const String& httpsFingerprint, bool reboot) __attribute__((deprecated));
-    t_httpUpdate_return updateSpiffs(const String& url, const String& currentVersion = "") __attribute__((deprecated));
-    t_httpUpdate_return updateSpiffs(const String& url, const String& currentVersion, const String& httpsFingerprint) __attribute__((deprecated));
-    t_httpUpdate_return updateSpiffs(const String& url, const String& currentVersion, const uint8_t httpsFingerprint[20]) __attribute__((deprecated)); // BearSSL
-#endif
     t_httpUpdate_return updateFS(WiFiClient& client, const String& url, const String& currentVersion = "");
-    t_httpUpdate_return updateSpiffs(WiFiClient& client, const String& url, const String& currentVersion = "") __attribute__((deprecated)) {
-        return updateFS(client, url, currentVersion);
-    };
 
     // Notification callbacks
     void onStart(HTTPUpdateStartCB cbOnStart)          { _cbStart = cbOnStart; }
@@ -174,9 +139,13 @@ protected:
     int _lastError;
     bool _rebootOnUpdate = true;
     bool _closeConnectionsOnUpdate = true;
+    String _user;
+    String _password;
+    String _auth;
+
 private:
     int _httpClientTimeout;
-    followRedirects_t _followRedirects;
+    followRedirects_t _followRedirects = HTTPC_DISABLE_FOLLOW_REDIRECTS;
 
     // Callbacks
     HTTPUpdateStartCB    _cbStart;
@@ -184,7 +153,7 @@ private:
     HTTPUpdateErrorCB    _cbError;
     HTTPUpdateProgressCB _cbProgress;
 
-    int _ledPin;
+    int _ledPin = -1;
     uint8_t _ledOn;
 };
 
