@@ -8,13 +8,13 @@
     Essentially, this is an rewrite of the original EPS8266 Multicast DNS code (ESP8266mDNS).
     The target of this rewrite was to keep the existing interface as stable as possible while
     adding and extending the supported set of mDNS features.
-    A lot of the additions were basicly taken from Erik Ekman's lwIP mdns app code.
+    A lot of the additions were basically taken from Erik Ekman's lwIP mdns app code.
 
     Supported mDNS features (in some cases somewhat limited):
     - Presenting a DNS-SD service to interested observers, eg. a http server by presenting _http._tcp service
     - Support for multi-level compressed names in input; in output only a very simple one-leven full-name compression is implemented
     - Probing host and service domains for uniqueness in the local network
-    - Tiebreaking while probing is supportet in a very minimalistic way (the 'higher' IP address wins the tiebreak)
+    - Tiebreaking while probing is supported in a very minimalistic way (the 'higher' IP address wins the tiebreak)
     - Announcing available services after successful probing
     - Using fixed service TXT items or
     - Using dynamic service TXT items for presented services (via callback)
@@ -52,7 +52,7 @@
     Static:
       Call 'uint32_t u32AnswerCount = MDNS.queryService("http", "tcp");'
       Iterate answers by: 'for (uint32_t u=0; u<u32AnswerCount; ++u) { const char* pHostname = MDNS.answerHostname(u); }'
-      You should call MDNS.removeQuery() sometimes later (when the answers are nott needed anymore)
+      You should call MDNS.removeQuery() sometimes later (when the answers are not needed anymore)
 
     Dynamic:
       Install a dynamic query by calling 'DNSResponder::hMDNSServiceQuery hServiceQuery = MDNS.installServiceQuery("http", "tcp", serviceQueryCallback, &userData);'
@@ -130,8 +130,9 @@ namespace MDNSImplementation
 #endif
 
 #define MDNS_IP4_SUPPORT
+#if LWIP_IPV6
 //#define MDNS_IP6_SUPPORT
-
+#endif
 
 #ifdef MDNS_IP4_SUPPORT
 #define MDNS_IP4_SIZE               4
@@ -164,6 +165,10 @@ namespace MDNSImplementation
 */
 #define MDNS_QUERYSERVICES_WAIT_TIME    1000
 
+/*
+    Timeout for udpContext->sendtimeout()
+*/
+#define MDNS_UDPCONTEXT_TIMEOUT  50
 
 /**
     MDNSResponder
@@ -172,6 +177,7 @@ class MDNSResponder
 {
 public:
     /* INTERFACE */
+
     MDNSResponder(void);
     virtual ~MDNSResponder(void);
 
@@ -184,10 +190,12 @@ public:
     {
         return begin(p_strHostname.c_str(), p_IPAddress, p_u32TTL);
     }
+    bool _joinMulticastGroups(void);
+    bool _leaveMulticastGroups(void);
 
     // Finish MDNS processing
     bool close(void);
-    // for esp32 compatability
+    // for esp32 compatibility
     bool end(void);
     // Change hostname (probing is restarted)
     bool setHostname(const char* p_pcHostname);
@@ -233,7 +241,7 @@ public:
     {
         setHostname(p_pcHostname);
     }
-    // for esp32 compatibilty
+    // for esp32 compatibility
     void setInstanceName(const String& s_pcHostname)
     {
         setInstanceName(s_pcHostname.c_str());
@@ -1172,7 +1180,7 @@ protected:
         uint8_t                 m_u8HostReplyMask;          // Flags for reply components/answers
         bool                    m_bLegacyQuery;             // Flag: Legacy query
         bool                    m_bResponse;                // Flag: Response to a query
-        bool                    m_bAuthorative;             // Flag: Authorative (owner) response
+        bool                    m_bAuthorative;             // Flag: Authoritative (owner) response
         bool                    m_bCacheFlush;              // Flag: Clients should flush their caches
         bool                    m_bUnicast;                 // Flag: Unicast response
         bool                    m_bUnannounce;              // Flag: Unannounce service
@@ -1183,6 +1191,7 @@ protected:
         ~stcMDNSSendParameter(void);
 
         bool clear(void);
+        bool clearCachedNames(void);
 
         bool shiftOffset(uint16_t p_u16Shift);
 
@@ -1198,12 +1207,8 @@ protected:
     UdpContext*                     m_pUDPContext;
     char*                           m_pcHostname;
     stcMDNSServiceQuery*            m_pServiceQueries;
-    WiFiEventHandler                m_DisconnectedHandler;
-    WiFiEventHandler                m_GotIPHandler;
     MDNSDynamicServiceTxtCallbackFunc m_fnServiceTxtCallback;
-    bool                            m_bPassivModeEnabled;
     stcProbeInformation             m_HostProbeInformation;
-    CONST netif*                    m_netif; // network interface to run on
 
     /** CONTROL **/
     /* MAINTENANCE */
@@ -1258,11 +1263,6 @@ protected:
                         uint16_t p_u16QueryType,
                         stcMDNSServiceQuery::stcAnswer* p_pKnownAnswers = 0);
 
-    const IPAddress _getResponseMulticastInterface() const
-    {
-        return IPAddress(m_netif->ip_addr);
-    }
-
     uint8_t _replyMaskForHost(const stcMDNS_RRHeader& p_RRHeader,
                               bool* p_pbFullNameMatch = 0) const;
     uint8_t _replyMaskForService(const stcMDNS_RRHeader& p_RRHeader,
