@@ -37,6 +37,7 @@ extern "C" {
 #include "user_interface.h"
 }
 
+#include "ESP8266mDNS.h"
 #include "LEAmDNS_lwIPdefs.h"
 #include "LEAmDNS_Priv.h"
 
@@ -84,10 +85,8 @@ bool MDNSResponder::_process(bool p_bUserContext)
     }
     else
     {
-        bResult = (m_netif != nullptr) &&
-                  (m_netif->flags & NETIF_FLAG_UP) &&  // network interface is up and running
-                  _updateProbeStatus() &&              // Probing
-                  _checkServiceQueryCache();           // Service query cache check
+        bResult =  _updateProbeStatus() &&              // Probing
+                   _checkServiceQueryCache();           // Service query cache check
     }
     return bResult;
 }
@@ -98,13 +97,10 @@ bool MDNSResponder::_process(bool p_bUserContext)
 bool MDNSResponder::_restart(void)
 {
 
-    return ((m_netif != nullptr) &&
-            (m_netif->flags & NETIF_FLAG_UP) &&  // network interface is up and running
-            (_resetProbeStatus(true)) &&         // Stop and restart probing
-            (_allocUDPContext()));               // Restart UDP
+    return ((_resetProbeStatus(true/*restart*/)) &&  // Stop and restart probing
+            (_allocUDPContext()));                   // Restart UDP
 }
 
-
 /**
     RECEIVING
 */
@@ -191,8 +187,7 @@ bool MDNSResponder::_parseQuery(const MDNSResponder::stcMDNS_MsgHeader& p_MsgHea
         {
             // Define host replies, BUT only answer queries after probing is done
             u8HostOrServiceReplies =
-                sendParameter.m_u8HostReplyMask |= (((m_bPassivModeEnabled) ||
-                                                    (ProbingStatus_Done == m_HostProbeInformation.m_ProbingStatus))
+                sendParameter.m_u8HostReplyMask |= (((ProbingStatus_Done == m_HostProbeInformation.m_ProbingStatus))
                                                     ? _replyMaskForHost(questionRR.m_Header, 0)
                                                     : 0);
             DEBUG_EX_INFO(if (u8HostOrServiceReplies)
@@ -221,8 +216,7 @@ bool MDNSResponder::_parseQuery(const MDNSResponder::stcMDNS_MsgHeader& p_MsgHea
             for (stcMDNSService* pService = m_pServices; pService; pService = pService->m_pNext)
             {
                 // Define service replies, BUT only answer queries after probing is done
-                uint8_t u8ReplyMaskForQuestion = (((m_bPassivModeEnabled) ||
-                                                   (ProbingStatus_Done == pService->m_ProbeInformation.m_ProbingStatus))
+                uint8_t u8ReplyMaskForQuestion = (((ProbingStatus_Done == pService->m_ProbeInformation.m_ProbingStatus))
                                                   ? _replyMaskForService(questionRR.m_Header, *pService, 0)
                                                   : 0);
                 u8HostOrServiceReplies |= (pService->m_u8ReplyMask |= u8ReplyMaskForQuestion);
@@ -363,9 +357,8 @@ bool MDNSResponder::_parseQuery(const MDNSResponder::stcMDNS_MsgHeader& p_MsgHea
                         // IP4 address was asked for
 #ifdef MDNS_IP4_SUPPORT
                         if ((AnswerType_A == pKnownRRAnswer->answerType()) &&
-                                (((stcMDNS_RRAnswerA*)pKnownRRAnswer)->m_IPAddress == _getResponseMulticastInterface()))
+                                (((stcMDNS_RRAnswerA*)pKnownRRAnswer)->m_IPAddress == m_pUDPContext->getInputNetif()->ip_addr))
                         {
-
                             DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _parseQuery: IP4 address already known... skipping!\n")););
                             sendParameter.m_u8HostReplyMask &= ~ContentFlag_A;
                         }   // else: RData NOT IP4 length !!
@@ -398,10 +391,11 @@ bool MDNSResponder::_parseQuery(const MDNSResponder::stcMDNS_MsgHeader& p_MsgHea
 #ifdef MDNS_IP4_SUPPORT
                         if (AnswerType_A == pKnownRRAnswer->answerType())
                         {
-                            IPAddress   localIPAddress(_getResponseMulticastInterface());
+
+                            IPAddress localIPAddress(m_pUDPContext->getInputNetif()->ip_addr);
                             if (((stcMDNS_RRAnswerA*)pKnownRRAnswer)->m_IPAddress == localIPAddress)
                             {
-                                // SAME IP address -> We've received an old message from ourselfs (same IP)
+                                // SAME IP address -> We've received an old message from ourselves (same IP)
                                 DEBUG_EX_RX(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _parseQuery: Tiebreak (IP4) WON (was an old message)!\n")););
                                 m_HostProbeInformation.m_bTiebreakNeeded = false;
                             }
@@ -416,7 +410,7 @@ bool MDNSResponder::_parseQuery(const MDNSResponder::stcMDNS_MsgHeader& p_MsgHea
                                 }
                                 else    // WON tiebreak
                                 {
-                                    //TiebreakState = TiebreakState_Won;    // We received an 'old' message from ourselfs -> Just ignore
+                                    //TiebreakState = TiebreakState_Won;    // We received an 'old' message from ourselves -> Just ignore
                                     DEBUG_EX_RX(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _parseQuery: Tiebreak (IP4) WON (higher IP)!\n")););
                                     m_HostProbeInformation.m_bTiebreakNeeded = false;
                                 }
@@ -507,7 +501,7 @@ bool MDNSResponder::_parseQuery(const MDNSResponder::stcMDNS_MsgHeader& p_MsgHea
                                         (hostDomain == ((stcMDNS_RRAnswerSRV*)pKnownRRAnswer)->m_SRVDomain))    // Host domain match
                                 {
 
-                                    // We've received an old message from ourselfs (same SRV)
+                                    // We've received an old message from ourselves (same SRV)
                                     DEBUG_EX_RX(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _parseQuery: Tiebreak (SRV) won (was an old message)!\n")););
                                     pService->m_ProbeInformation.m_bTiebreakNeeded = false;
                                 }
@@ -522,7 +516,7 @@ bool MDNSResponder::_parseQuery(const MDNSResponder::stcMDNS_MsgHeader& p_MsgHea
                                     }
                                     else    // WON tiebreak
                                     {
-                                        //TiebreakState = TiebreakState_Won;    // We received an 'old' message from ourselfs -> Just ignore
+                                        //TiebreakState = TiebreakState_Won;    // We received an 'old' message from ourselves -> Just ignore
                                         DEBUG_EX_RX(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _parseQuery: Tiebreak (SRV) won (higher)!\n")););
                                         pService->m_ProbeInformation.m_bTiebreakNeeded = false;
                                     }
@@ -1220,9 +1214,7 @@ bool MDNSResponder::_updateProbeStatus(void)
 
     //
     // Probe host domain
-    if ((ProbingStatus_ReadyToStart == m_HostProbeInformation.m_ProbingStatus) &&                   // Ready to get started AND
-            //TODO: Fix the following to allow Ethernet shield or other interfaces
-            (_getResponseMulticastInterface() != IPAddress()))                // Has IP address
+    if (ProbingStatus_ReadyToStart == m_HostProbeInformation.m_ProbingStatus)
     {
         DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _updateProbeStatus: Starting host probing...\n")););
 
@@ -1568,7 +1560,7 @@ bool MDNSResponder::_announce(bool p_bAnnounce,
 
         bResult = true;
 
-        sendParameter.m_bResponse = true;           // Announces are 'Unsolicited authorative responses'
+        sendParameter.m_bResponse = true;           // Announces are 'Unsolicited authoritative responses'
         sendParameter.m_bAuthorative = true;
         sendParameter.m_bUnannounce = !p_bAnnounce; // When unannouncing, the TTL is set to '0' while creating the answers
 
@@ -1620,7 +1612,7 @@ bool MDNSResponder::_announceService(stcMDNSService& p_rService,
     if (ProbingStatus_Done == p_rService.m_ProbeInformation.m_ProbingStatus)
     {
 
-        sendParameter.m_bResponse = true;           // Announces are 'Unsolicited authorative responses'
+        sendParameter.m_bResponse = true;           // Announces are 'Unsolicited authoritative responses'
         sendParameter.m_bAuthorative = true;
         sendParameter.m_bUnannounce = !p_bAnnounce; // When unannouncing, the TTL is set to '0' while creating the answers
 
@@ -1982,7 +1974,7 @@ bool MDNSResponder::_checkServiceQueryCache(void)
 /*
     MDNSResponder::_replyMaskForHost
 
-    Determines the relavant host answers for the given question.
+    Determines the relevant host answers for the given question.
     - A question for the hostname (eg. esp8266.local) will result in an A/AAAA (eg. 192.168.2.129) reply.
     - A question for the reverse IP address (eg. 192-168.2.120.inarpa.arpa) will result in an PTR_IP4 (eg. esp8266.local) reply.
 
@@ -2006,11 +1998,17 @@ uint8_t MDNSResponder::_replyMaskForHost(const MDNSResponder::stcMDNS_RRHeader&
             // PTR request
 #ifdef MDNS_IP4_SUPPORT
             stcMDNS_RRDomain    reverseIP4Domain;
-            if ((_buildDomainForReverseIP4(_getResponseMulticastInterface(), reverseIP4Domain)) &&
-                    (p_RRHeader.m_Domain == reverseIP4Domain))
+            for (netif* pNetIf = netif_list; pNetIf; pNetIf = pNetIf->next)
             {
-                // Reverse domain match
-                u8ReplyMask |= ContentFlag_PTR_IP4;
+                if (netif_is_up(pNetIf))
+                {
+                    if ((_buildDomainForReverseIP4(pNetIf->ip_addr, reverseIP4Domain)) &&
+                            (p_RRHeader.m_Domain == reverseIP4Domain))
+                    {
+                        // Reverse domain match
+                        u8ReplyMask |= ContentFlag_PTR_IP4;
+                    }
+                }
             }
 #endif
 #ifdef MDNS_IP6_SUPPORT
