@@ -26,6 +26,7 @@ extern "C" {
 #include "user_interface.h"
 }
 
+#include "ESP8266mDNS.h"
 #include "LEAmDNS_lwIPdefs.h"
 #include "LEAmDNS_Priv.h"
 
@@ -88,8 +89,8 @@ bool MDNSResponder::_sendMDNSMessage(MDNSResponder::stcMDNSSendParameter& p_rSen
         });
         IPAddress   ipRemote;
         ipRemote = m_pUDPContext->getRemoteAddress();
-        bResult = ((_prepareMDNSMessage(p_rSendParameter, _getResponseMulticastInterface())) &&
-                   (m_pUDPContext->send(ipRemote, m_pUDPContext->getRemotePort())));
+        bResult = ((_prepareMDNSMessage(p_rSendParameter, m_pUDPContext->getInputNetif()->ip_addr)) &&
+                   (m_pUDPContext->sendTimeout(ipRemote, m_pUDPContext->getRemotePort(), MDNS_UDPCONTEXT_TIMEOUT)));
     }
     else                                // Multicast response
     {
@@ -120,25 +121,32 @@ bool MDNSResponder::_sendMDNSMessage_Multicast(MDNSResponder::stcMDNSSendParamet
 
     bool    bResult = false;
 
-    IPAddress   fromIPAddress;
-    fromIPAddress = _getResponseMulticastInterface();
-    m_pUDPContext->setMulticastInterface(fromIPAddress);
+    for (netif* pNetIf = netif_list; pNetIf; pNetIf = pNetIf->next)
+    {
+        if (netif_is_up(pNetIf))
+        {
+            IPAddress   fromIPAddress;
+            //fromIPAddress = _getResponseMulticastInterface();
+            fromIPAddress = pNetIf->ip_addr;
+            m_pUDPContext->setMulticastInterface(fromIPAddress);
 
 #ifdef MDNS_IP4_SUPPORT
-    IPAddress   toMulticastAddress(DNS_MQUERY_IPV4_GROUP_INIT);
+            IPAddress   toMulticastAddress(DNS_MQUERY_IPV4_GROUP_INIT);
 #endif
 #ifdef MDNS_IP6_SUPPORT
-    //TODO: set multicast address
-    IPAddress   toMulticastAddress(DNS_MQUERY_IPV6_GROUP_INIT);
+            //TODO: set multicast address
+            IPAddress   toMulticastAddress(DNS_MQUERY_IPV6_GROUP_INIT);
 #endif
-    DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _sendMDNSMessage_Multicast: Will send to '%s'.\n"), toMulticastAddress.toString().c_str()););
-    bResult = ((_prepareMDNSMessage(p_rSendParameter, fromIPAddress)) &&
-               (m_pUDPContext->send(toMulticastAddress, DNS_MQUERY_PORT)));
+            DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _sendMDNSMessage_Multicast: Will send to '%s'.\n"), toMulticastAddress.toString().c_str()););
+            bResult = ((_prepareMDNSMessage(p_rSendParameter, fromIPAddress)) &&
+                       (m_pUDPContext->sendTimeout(toMulticastAddress, DNS_MQUERY_PORT, MDNS_UDPCONTEXT_TIMEOUT)));
 
-    DEBUG_EX_ERR(if (!bResult)
-{
-    DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _sendMDNSMessage_Multicast: FAILED!\n"));
-    });
+            DEBUG_EX_ERR(if (!bResult)
+        {
+            DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _sendMDNSMessage_Multicast: FAILED!\n"));
+            });
+        }
+    }
     return bResult;
 }
 
@@ -146,7 +154,7 @@ bool MDNSResponder::_sendMDNSMessage_Multicast(MDNSResponder::stcMDNSSendParamet
     MDNSResponder::_prepareMDNSMessage
 
     The MDNS message is composed in a two-step process.
-    In the first loop 'only' the header informations (mainly number of answers) are collected,
+    In the first loop 'only' the header information (mainly number of answers) are collected,
     while in the seconds loop, the header and all queries and answers are written to the UDP
     output buffer.
 
@@ -156,6 +164,7 @@ bool MDNSResponder::_prepareMDNSMessage(MDNSResponder::stcMDNSSendParameter& p_r
 {
     DEBUG_EX_INFO(DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _prepareMDNSMessage\n")););
     bool    bResult = true;
+    p_rSendParameter.clearCachedNames(); // Need to remove cached names, p_SendParameter might have been used before on other interface
 
     // Prepare header; count answers
     stcMDNS_MsgHeader  msgHeader(p_rSendParameter.m_u16ID, p_rSendParameter.m_bResponse, 0, p_rSendParameter.m_bAuthorative);
@@ -205,7 +214,7 @@ bool MDNSResponder::_prepareMDNSMessage(MDNSResponder::stcMDNSSendParameter& p_r
             DEBUG_EX_ERR(if (!bResult) DEBUG_OUTPUT.printf_P(PSTR("[MDNSResponder] _prepareMDNSMessage: _writeMDNSQuestion FAILED!\n")););
         }
 
-        // Answers and authorative answers
+        // Answers and authoritative answers
 #ifdef MDNS_IP4_SUPPORT
         if ((bResult) &&
                 (p_rSendParameter.m_u8HostReplyMask & ContentFlag_A))
@@ -388,7 +397,7 @@ bool MDNSResponder::_sendMDNSQuery(const MDNSResponder::stcMDNS_RRDomain& p_Quer
         // It seems, that some mDNS implementations don't support 'unicast response' questions...
         sendParameter.m_pQuestions->m_Header.m_Attributes.m_u16Class = (/*0x8000 |*/ DNS_RRCLASS_IN);   // /*Unicast &*/ INternet
 
-        // TODO: Add knwon answer to the query
+        // TODO: Add known answer to the query
         (void)p_pKnownAnswers;
 
         bResult = _sendMDNSMessage(sendParameter);
@@ -646,13 +655,13 @@ bool MDNSResponder::_readRRAnswerTXT(MDNSResponder::stcMDNS_RRAnswerTXT& p_rRRAn
                     }
 
                     if ((bResult) &&
-                            (pTxt))     // Everythings fine so far
+                            (pTxt))     // Everything is fine so far
                     {
                         // Link TXT item to answer TXTs
                         pTxt->m_pNext = p_rRRAnswerTXT.m_Txts.m_pTxts;
                         p_rRRAnswerTXT.m_Txts.m_pTxts = pTxt;
                     }
-                    else            // At least no TXT (migth be OK, if length was 0) OR an error
+                    else            // At least no TXT (might be OK, if length was 0) OR an error
                     {
                         if (!bResult)
                         {
@@ -1217,9 +1226,9 @@ bool MDNSResponder::_readMDNSMsgHeader(MDNSResponder::stcMDNS_MsgHeader& p_rMsgH
             (_udpRead16(p_rMsgHeader.m_u16ARCount)))
     {
 
-        p_rMsgHeader.m_1bQR     = (u8B1 & 0x80);    // Query/Responde flag
+        p_rMsgHeader.m_1bQR     = (u8B1 & 0x80);    // Query/Respond flag
         p_rMsgHeader.m_4bOpcode = (u8B1 & 0x78);    // Operation code (0: Standard query, others ignored)
-        p_rMsgHeader.m_1bAA     = (u8B1 & 0x04);    // Authorative answer
+        p_rMsgHeader.m_1bAA     = (u8B1 & 0x04);    // Authoritative answer
         p_rMsgHeader.m_1bTC     = (u8B1 & 0x02);    // Truncation flag
         p_rMsgHeader.m_1bRD     = (u8B1 & 0x01);    // Recursion desired
 
@@ -1421,7 +1430,7 @@ bool MDNSResponder::_writeMDNSServiceDomain(const MDNSResponder::stcMDNSService&
                        // Found cached domain -> mark as compressed domain
                        ? ((MDNS_DOMAIN_COMPRESS_MARK > ((u16CachedDomainOffset >> 8) & ~MDNS_DOMAIN_COMPRESS_MARK)) && // Valid offset
                           ((!p_bPrependRDLength) ||
-                           (_write16(2, p_rSendParameter))) &&                                     // Lenght of 'Cxxx'
+                           (_write16(2, p_rSendParameter))) &&                                     // Length of 'Cxxx'
                           (_write8(((u16CachedDomainOffset >> 8) | MDNS_DOMAIN_COMPRESS_MARK), p_rSendParameter)) &&   // Compression mark (and offset)
                           (_write8((uint8_t)(u16CachedDomainOffset & 0xFF), p_rSendParameter)))
                        // No cached domain -> add this domain to cache and write full domain name
