@@ -51,7 +51,7 @@ bool SoftwareSerial::isValidGPIOpin(int8_t pin) {
     return (pin >= 0 && pin <= 16) && !isFlashInterfacePin(pin);
 #elif defined(ESP32)
     return (pin >= 0 && pin <= 5) || (pin >= 12 && pin <= 19) ||
-        (pin >= 21 && pin <= 23) || (pin >= 25 && pin <= 27) || (pin >= 32 && pin <= 35);
+        (pin >= 21 && pin <= 23) || (pin >= 25 && pin <= 27) || (pin >= 32 && pin <= 39);
 #else
     return true;
 #endif
@@ -65,6 +65,23 @@ bool SoftwareSerial::isValidRxGPIOpin(int8_t pin) {
         ;
 }
 
+bool SoftwareSerial::isValidTxGPIOpin(int8_t pin) {
+    return isValidGPIOpin(pin)
+#if defined(ESP32)
+        && (pin < 34)
+#endif
+        ;
+}
+
+bool SoftwareSerial::hasRxGPIOPullUp(int8_t pin) {
+#if defined(ESP32)
+    return !(pin >= 34 && pin <= 39);
+#else
+    (void)pin;
+    return true;
+#endif
+}
+
 void SoftwareSerial::begin(uint32_t baud, SoftwareSerialConfig config,
     int8_t rxPin, int8_t txPin,
     bool invert, int bufCapacity, int isrBufCapacity) {
@@ -85,14 +102,14 @@ void SoftwareSerial::begin(uint32_t baud, SoftwareSerialConfig config,
             m_parityBuffer.reset(new circular_queue<uint8_t>((m_buffer->capacity() + 7) / 8));
             m_parityInPos = m_parityOutPos = 1;
         }
-        m_isrBuffer.reset(new circular_queue<uint32_t>((isrBufCapacity > 0) ?
+        m_isrBuffer.reset(new circular_queue<uint32_t, SoftwareSerial*>((isrBufCapacity > 0) ?
             isrBufCapacity : m_buffer->capacity() * (2 + m_dataBits + static_cast<bool>(m_parityMode))));
         if (m_buffer && (!m_parityMode || m_parityBuffer) && m_isrBuffer) {
             m_rxValid = true;
-            pinMode(m_rxPin, INPUT_PULLUP);
+            pinMode(m_rxPin, hasRxGPIOPullUp(m_rxPin) ? INPUT_PULLUP : INPUT);
         }
     }
-    if (isValidGPIOpin(m_txPin)) {
+    if (isValidTxGPIOpin(m_txPin)) {
         m_txValid = true;
         if (!m_oneWire) {
             pinMode(m_txPin, OUTPUT);
@@ -120,7 +137,7 @@ uint32_t SoftwareSerial::baudRate() {
 }
 
 void SoftwareSerial::setTransmitEnablePin(int8_t txEnablePin) {
-    if (isValidGPIOpin(txEnablePin)) {
+    if (isValidTxGPIOpin(txEnablePin)) {
         m_txEnableValid = true;
         m_txEnablePin = txEnablePin;
         pinMode(m_txEnablePin, OUTPUT);
@@ -143,7 +160,7 @@ void SoftwareSerial::enableTx(bool on) {
             digitalWrite(m_txPin, !m_invert);
         }
         else {
-            pinMode(m_rxPin, INPUT_PULLUP);
+            pinMode(m_rxPin, hasRxGPIOPullUp(m_rxPin) ? INPUT_PULLUP : INPUT);
             enableRx(true);
         }
     }
@@ -187,9 +204,9 @@ int SoftwareSerial::read() {
     return val;
 }
 
-size_t SoftwareSerial::read(uint8_t* buffer, size_t size) {
+int SoftwareSerial::read(uint8_t* buffer, size_t size) {
     if (!m_rxValid) { return 0; }
-    size_t avail;
+    int avail;
     if (0 == (avail = m_buffer->pop_n(buffer, size))) {
         rxBits();
         avail = m_buffer->pop_n(buffer, size);
@@ -228,30 +245,31 @@ int SoftwareSerial::available() {
     return avail;
 }
 
-void ICACHE_RAM_ATTR SoftwareSerial::preciseDelay(bool sync) {
+void IRAM_ATTR SoftwareSerial::preciseDelay(bool sync) {
     if (!sync)
     {
         // Reenable interrupts while delaying to avoid other tasks piling up
         if (!m_intTxEnabled) { xt_wsr_ps(m_savedPS); }
-        auto expired = ESP.getCycleCount() - m_periodStart;
-        if (expired < m_periodDuration)
+        const auto expired = ESP.getCycleCount() - m_periodStart;
+        const int32_t remaining = m_periodDuration - expired;
+        const int32_t ms = remaining / 1000L / ESP.getCpuFreqMHz();
+        if (ms > 0)
         {
-            auto ms = (m_periodDuration - expired) / ESP.getCpuFreqMHz() / 1000UL;
-            if (ms) delay(ms);
+            delay(ms);
+        }
+        else
+        {
+            optimistic_yield(10000UL);
         }
-        while ((ESP.getCycleCount() - m_periodStart) < m_periodDuration) { optimistic_yield(10000UL); }
-        // Disable interrupts again
-        if (!m_intTxEnabled) { m_savedPS = xt_rsil(15); }
-    }
-    else
-    {
-        while ((ESP.getCycleCount() - m_periodStart) < m_periodDuration) {}
     }
+    while ((ESP.getCycleCount() - m_periodStart) < m_periodDuration) {}
+    // Disable interrupts again if applicable
+    if (!sync && !m_intTxEnabled) { m_savedPS = xt_rsil(15); }
     m_periodDuration = 0;
     m_periodStart = ESP.getCycleCount();
 }
 
-void ICACHE_RAM_ATTR SoftwareSerial::writePeriod(
+void IRAM_ATTR SoftwareSerial::writePeriod(
     uint32_t dutyCycle, uint32_t offCycle, bool withStopBit) {
     preciseDelay(true);
     if (dutyCycle)
@@ -280,7 +298,7 @@ size_t SoftwareSerial::write(const uint8_t* buffer, size_t size) {
     return write(buffer, size, m_parityMode);
 }
 
-size_t ICACHE_RAM_ATTR SoftwareSerial::write(const uint8_t* buffer, size_t size, SoftwareSerialParity parity) {
+size_t IRAM_ATTR SoftwareSerial::write(const uint8_t* buffer, size_t size, SoftwareSerialParity parity) {
     if (m_rxValid) { rxBits(); }
     if (!m_txValid) { return -1; }
 
@@ -300,7 +318,7 @@ size_t ICACHE_RAM_ATTR SoftwareSerial::write(const uint8_t* buffer, size_t size,
     m_periodDuration = 0;
     m_periodStart = ESP.getCycleCount();
     for (size_t cnt = 0; cnt < size; ++cnt) {
-        uint8_t byte = buffer[cnt] & dataMask;
+        uint8_t byte = pgm_read_byte(buffer + cnt) & dataMask;
         // push LSB start-data-parity-stop bit pattern into uint32_t
         // Stop bits: HIGH
         uint32_t word = ~0UL;
@@ -396,7 +414,6 @@ int SoftwareSerial::peek() {
 }
 
 void SoftwareSerial::rxBits() {
-    int isrAvail = m_isrBuffer->available();
 #ifdef ESP8266
     if (m_isrOverflow.load()) {
         m_overflow = true;
@@ -408,19 +425,20 @@ void SoftwareSerial::rxBits() {
     }
 #endif
 
-    // stop bit can go undetected if leading data bits are at same level
-    // and there was also no next start bit yet, so one byte may be pending.
-    // low-cost check first
-    if (!isrAvail && m_rxCurBit >= -1 && m_rxCurBit < m_pduBits - m_stopBits) {
-        uint32_t detectionCycles = (m_pduBits - m_stopBits - m_rxCurBit) * m_bitCycles;
-        if (ESP.getCycleCount() - m_isrLastCycle > detectionCycles) {
+    m_isrBuffer->for_each(m_isrBufferForEachDel);
+
+    // A stop bit can go undetected if leading data bits are at same level
+    // and there was also no next start bit yet, so one word may be pending.
+    // Check that there was no new ISR data received in the meantime, inserting an
+    // extraneous stop level bit out of sequence breaks rx.
+    if (m_rxCurBit > -1 && m_rxCurBit < m_pduBits - m_stopBits) {
+        const uint32_t detectionCycles = (m_pduBits - m_stopBits - m_rxCurBit) * m_bitCycles;
+        if (!m_isrBuffer->available() && ESP.getCycleCount() - m_isrLastCycle > detectionCycles) {
             // Produce faux stop bit level, prevents start bit maldetection
             // cycle's LSB is repurposed for the level bit
             rxBits(((m_isrLastCycle + detectionCycles) | 1) ^ m_invert);
         }
     }
-
-    m_isrBuffer->for_each([this](const uint32_t& isrCycle) { rxBits(isrCycle); });
 }
 
 void SoftwareSerial::rxBits(const uint32_t& isrCycle) {
@@ -500,7 +518,7 @@ void SoftwareSerial::rxBits(const uint32_t& isrCycle) {
     }
 }
 
-void ICACHE_RAM_ATTR SoftwareSerial::rxBitISR(SoftwareSerial* self) {
+void IRAM_ATTR SoftwareSerial::rxBitISR(SoftwareSerial* self) {
     uint32_t curCycle = ESP.getCycleCount();
     bool level = digitalRead(self->m_rxPin);
 
@@ -509,7 +527,7 @@ void ICACHE_RAM_ATTR SoftwareSerial::rxBitISR(SoftwareSerial* self) {
     if (!self->m_isrBuffer->push((curCycle | 1U) ^ !level)) self->m_isrOverflow.store(true);
 }
 
-void ICACHE_RAM_ATTR SoftwareSerial::rxBitSyncISR(SoftwareSerial* self) {
+void IRAM_ATTR SoftwareSerial::rxBitSyncISR(SoftwareSerial* self) {
     uint32_t start = ESP.getCycleCount();
     uint32_t wait = self->m_bitCycles - 172U;
 
