@@ -128,13 +128,17 @@ public:
     bool overflow();
 
     int available() override;
+#if defined(ESP8266)
+    int availableForWrite() override {
+#else
     int availableForWrite() {
+#endif
         if (!m_txValid) return 0;
         return 1;
     }
     int peek() override;
     int read() override;
-    /// @returns The verbatim parity bit associated with the last read() or peek() call
+    /// @returns The verbatim parity bit associated with the last successful read() or peek() call
     bool readParity()
     {
         return m_lastReadParity;
@@ -152,9 +156,13 @@ public:
         return (0x9669 >> byte) & 1;
     }
     /// The read(buffer, size) functions are non-blocking, the same as readBytes but without timeout
-    size_t read(uint8_t* buffer, size_t size);
+    int read(uint8_t* buffer, size_t size)
+#if defined(ESP8266)
+        override
+#endif
+        ;
     /// The read(buffer, size) functions are non-blocking, the same as readBytes but without timeout
-    size_t read(char* buffer, size_t size) {
+    int read(char* buffer, size_t size) {
         return read(reinterpret_cast<uint8_t*>(buffer), size);
     }
     /// @returns The number of bytes read into buffer, up to size. Times out if the limit set through
@@ -211,6 +219,9 @@ private:
         uint32_t dutyCycle, uint32_t offCycle, bool withStopBit);
     bool isValidGPIOpin(int8_t pin);
     bool isValidRxGPIOpin(int8_t pin);
+    bool isValidTxGPIOpin(int8_t pin);
+    // result is only defined for a valid Rx GPIO pin
+    bool hasRxGPIOPullUp(int8_t pin);
     /* check m_rxValid that calling is safe */
     void rxBits();
     void rxBits(const uint32_t& isrCycle);
@@ -248,7 +259,8 @@ private:
     uint32_t m_savedPS = 0;
     // the ISR stores the relative bit times in the buffer. The inversion corrected level is used as sign bit (2's complement):
     // 1 = positive including 0, 0 = negative.
-    std::unique_ptr<circular_queue<uint32_t> > m_isrBuffer;
+    std::unique_ptr<circular_queue<uint32_t, SoftwareSerial*> > m_isrBuffer;
+    const Delegate<void(uint32_t&&), SoftwareSerial*> m_isrBufferForEachDel = { [](SoftwareSerial* self, uint32_t&& isrCycle) { self->rxBits(isrCycle); }, this };
     std::atomic<bool> m_isrOverflow;
     uint32_t m_isrLastCycle;
     bool m_rxCurParity = false;
