@@ -13,7 +13,13 @@ test -d libraries
 all="
 libraries/ESP8266mDNS
 libraries/Wire
+libraries/lwIP*
+cores/esp8266/Lwip*
+cores/esp8266/debug*
 cores/esp8266/core_esp8266_si2c.cpp
+cores/esp8266/StreamString.*
+cores/esp8266/StreamSend.*
+libraries/Netdump
 "
 
 # core
