@@ -54,7 +54,6 @@ while true; do
 Which build?
 1. main
 2. main + IPv6
-3. main with lwIP-v1.4
 4. debug even
 5. debug odd
 6. platformio
@@ -69,7 +68,6 @@ EOF
 	case "$ans" in
 		1) BUILD_TYPE=build;;
 		2) BUILD_TYPE=build6;;
-		3) BUILD_TYPE=build1;;
 		4) BUILD_TYPE=debug_even;;
 		5) BUILD_TYPE=debug_odd;;
 		6) BUILD_TYPE=platformio;;
@@ -106,13 +104,6 @@ elif [ "$BUILD_TYPE" = "build6_even" ]; then
 elif [ "$BUILD_TYPE" = "build6_odd" ]; then
     BUILD_PARITY=odd tests/build6.sh
 
-elif [ "$BUILD_TYPE" = "build1" ]; then
-    tests/build1.sh
-elif [ "$BUILD_TYPE" = "build1_even" ]; then
-    BUILD_PARITY=even tests/build1.sh
-elif [ "$BUILD_TYPE" = "build1_odd" ]; then
-    BUILD_PARITY=odd tests/build1.sh
-
 elif [ "$BUILD_TYPE" = "platformio" ]; then
     tests/platformio.sh
 elif [ "$BUILD_TYPE" = "platformio_even" ]; then
