@@ -77,7 +77,7 @@ def serve(remoteAddr, localAddr, remotePort, localPort, password, filename, comm
   try:
     sock.bind(server_address)
     sock.listen(1)
-  except:
+  except Exception:
     logging.error("Listen Failed")
     return 1
 
@@ -100,11 +100,11 @@ def serve(remoteAddr, localAddr, remotePort, localPort, password, filename, comm
   logging.info('Sending invitation to: %s', remoteAddr)
   sock2 = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
   remote_address = (remoteAddr, int(remotePort))
-  sent = sock2.sendto(message.encode(), remote_address)
+  sock2.sendto(message.encode(), remote_address)
   sock2.settimeout(10)
   try:
     data = sock2.recv(128).decode()
-  except:
+  except Exception:
     logging.error('No Answer')
     sock2.close()
     return 1
@@ -123,7 +123,7 @@ def serve(remoteAddr, localAddr, remotePort, localPort, password, filename, comm
       sock2.settimeout(10)
       try:
         data = sock2.recv(32).decode()
-      except:
+      except Exception:
         sys.stderr.write('FAIL\n')
         logging.error('No Answer to our Authentication')
         sock2.close()
@@ -147,7 +147,7 @@ def serve(remoteAddr, localAddr, remotePort, localPort, password, filename, comm
     connection, client_address = sock.accept()
     sock.settimeout(None)
     connection.settimeout(None)
-  except:
+  except Exception:
     logging.error('No response from device')
     sock.close()
     return 1
@@ -173,7 +173,7 @@ def serve(remoteAddr, localAddr, remotePort, localPort, password, filename, comm
         if connection.recv(32).decode().find('O') >= 0:
           # connection will receive only digits or 'OK'
           received_ok = True
-      except:
+      except Exception:
         sys.stderr.write('\n')
         logging.error('Error Uploading')
         connection.close()
@@ -207,7 +207,7 @@ def serve(remoteAddr, localAddr, remotePort, localPort, password, filename, comm
       if received_ok:
         return 0
       return 1
-    except:
+    except Exception:
       logging.error('No Result!')
       connection.close()
       f.close()
