@@ -51,8 +51,15 @@ author: d. gauchard
 #include "lwip-git.h"
 
 // this is dhcpserver taken from lwip-1.4-espressif
+// arduino will use its own sources based on the above
+// this source only needs dhcps_start() prototype
 #include "lwip/apps-esp/dhcpserver.h"
+
 // this is espconn taken from lwip-1.4-espressif
+// espconn is probably a modified version of lwIP's netconn
+// note:
+//  arduino needs to call espconn_init()
+//  esp-open-sdk always calls it
 #include "lwip/apps-esp/espconn.h"
 
 #define DBG "GLUE: "
@@ -61,6 +68,7 @@ author: d. gauchard
 //#define netif_ap  (&netif_git[SOFTAP_IF])
 
 struct netif netif_git[2];
+int netif_enabled[2] = { 0, 0 };
 const char netif_name[2][8] = { "station", "soft-ap" };
 
 int __attribute__((weak)) doprint_allow = 0; // for doprint()
@@ -483,6 +491,13 @@ void esp2glue_netif_set_up1down0 (int netif_idx, int up1_or_down0)
 		netif_set_link_up(netif);
 		//netif_set_up(netif); // unwanted call to netif_sta_status_callback()
 		netif->flags |= NETIF_FLAG_UP;
+#if ARDUINO
+		if (!netif_enabled[netif_idx])
+		{
+		    netif_enabled[netif_idx] = 1;
+		    netif_status_changed(netif);
+		}
+#endif
 	}
 	else
 	{
@@ -501,6 +516,13 @@ void esp2glue_netif_set_up1down0 (int netif_idx, int up1_or_down0)
 
 		if (netif_default == &netif_git[netif_idx])
 			netif_set_default(NULL);
+#if ARDUINO
+		if (netif_enabled[netif_idx])
+		{
+		    netif_enabled[netif_idx] = 0;
+		    netif_status_changed(netif);
+		}
+#endif
 	}
 }
 
@@ -518,3 +540,33 @@ LWIP_ERR_T lwip_unhandled_packet (struct pbuf* pbuf, struct netif* netif)
 	(void)netif;
 	return ERR_ARG;
 }
+
+#if ARDUINO
+
+void netif_status_changed (struct netif*) __attribute__((weak));
+void netif_status_changed (struct netif* netif)
+{
+    (void)netif;
+}
+
+void lwip_hook_dhcp_parse_option(struct netif *netif, struct dhcp *dhcp, int state, struct dhcp_msg *msg,
+                                 int msg_type, int option, int option_len, struct pbuf *pbuf,
+                                 int option_value_offset) __attribute__((weak));
+
+void lwip_hook_dhcp_parse_option(struct netif *netif, struct dhcp *dhcp, int state, struct dhcp_msg *msg,
+                                 int msg_type, int option, int option_len, struct pbuf *pbuf,
+                                 int option_value_offset)
+{
+    (void)netif;
+    (void)dhcp;
+    (void)state;
+    (void)msg;
+    (void)msg_type;
+    (void)option;
+    (void)option_len;
+    (void)pbuf;
+    (void)option_value_offset;
+    uprint(DBG "unhandled dhcp option in lwip_hook_dhcp_parse_option()\n");
+}
+
+#endif // ARDUINO
